# $Id: rules.mk,v 1.32 2000/04/01 12:27:01 mark Exp steve $
all: 

debug:
	${MAKE} all "DBGFLAGS=-g"

nodebug:
	${MAKE} all "DBGFLAGS="


initprj:
	@if [ "X${NEWDIR}X" = XX ] ; then \
		echo " "; \
		echo "Must specify a NEWDIR name " ; \
		echo " "; \
		echo "Usage: " ; \
		echo "  make initprj NEWDIR=projname" ; \
		echo " "; \
		exit 3 ; \
	fi;
	@if [ -d ${NEWDIR} ] ; then \
		echo " "; \
		echo Project ${NEWDIR} already exists ; \
		echo " "; \
		exit 3 ; \
	fi;
	${MKDIR} ${NEWDIR}
	@${CHMOD} g+ws ${NEWDIR}
	${MKDIR} ${NEWDIR}/RCS
	@${CHMOD} g+ws ${NEWDIR}/RCS
	${INSTALL} ./Makefile ${NEWDIR}
	@echo "Removing unneeded dependency file"
	${RM} -f ./depend.mk


distclean: clean
	${RM} -f ${XFILES} ${TESTXFILES}


clean:
	${RM} -f *.o *.a core depend.mk

install:
	@echo "Use either install-dev, install-stable or install-all"

install-prod:
	@echo "install-prod is now obsolete, use install-stable instead"

install-all:
	@${MAKE} install-dev
	@${MAKE} install-stable

install-dev:
	@if [ "X${PROJNAME}X" = XX ] ; then \
		echo " "; \
		echo "No PROJNAME defined in the Makefile" ; \
		echo "    ... aborting install" ; \
		echo " "; \
		exit 3 ; \
	fi;
	@${MAKE} all
	@${MAKE} hdrdevinstall
	@${MAKE} libdevinstall

install-stable:
	@if [ "X${PROJNAME}X" = XX ] ; then \
		echo " " ; \
		echo "No PROJNAME defined in the Makefile" ; \
		echo "    ... aborting install" ; \
		echo " " ; \
		exit 4 ; \
	fi;
	@${MAKE} all
	@${MAKE} hdrinstall
	@${MAKE} libinstall
	@${MAKE} exeinstall
	@${MAKE} tclinstall
	@${MAKE} docinstall
	@${MAKE} refdocinstall
	@${MAKE} maninstall
	@${MAKE} srcinstall


maninstall:
	@echo " "
	@echo "Installing man pages"
	@if [ ! -d ${MANDIR} ] ; then \
		${MKDIR} ${MANDIR} ; \
		${CHMOD} g+w ${MANDIR} ; \
		${MKDIR} ${MANDIR}/man1 ; \
		${CHMOD} g+w ${MANDIR}/man1 ; \
		${MKDIR} ${MANDIR}/man2 ; \
		${CHMOD} g+w ${MANDIR}/man2 ; \
		${MKDIR} ${MANDIR}/man3 ; \
		${CHMOD} g+w ${MANDIR}/man3 ; \
		${MKDIR} ${MANDIR}/man4 ; \
		${CHMOD} g+w ${MANDIR}/man4 ; \
		${MKDIR} ${MANDIR}/man5 ; \
		${CHMOD} g+w ${MANDIR}/man5 ; \
		${MKDIR} ${MANDIR}/man6 ; \
		${CHMOD} g+w ${MANDIR}/man6 ; \
		${MKDIR} ${MANDIR}/man7 ; \
		${CHMOD} g+w ${MANDIR}/man7 ; \
		${MKDIR} ${MANDIR}/man8 ; \
		${CHMOD} g+w ${MANDIR}/man8 ; \
		${MKDIR} ${MANDIR}/mann ; \
		${CHMOD} g+w ${MANDIR}/mann ; \
		${MKDIR} ${MANDIR}/manl ; \
		${CHMOD} g+w ${MANDIR}/manl ; \
		fi
	@for mansec in n l 1 2 3 4 5 6 7 8 ; do \
	   for manfile in doc/*.$$mansec refdoc/*.$$mansec verylongdummyname ; do \
		if [ -f $$manfile ] ; then \
			${INSTALL} -m 0664 $$manfile ${MANDIR}/man$$mansec ; \
			echo ${INSTALL} -m 0664 $$manfile ${MANDIR}/man$$mansec ; \
		fi \
	   done \
	done
	@if ${CHMOD} -R ug+w ${MANDIR} ; then \
		echo "${CHMOD} -R ug+w ${MANDIR}" ; \
	fi


docinstall:
	@echo " "
	@echo "Installing (userguide) documents"
	@if [ ! -d ${DOCDIR} ] ; then \
		${MKDIR} ${DOCDIR} ; \
		${CHMOD} g+w ${DOCDIR} ; \
		fi
	@${RM} -r -f ${DOCDIR}/${PROJNAME}
	@${MKDIR} ${DOCDIR}/${PROJNAME}
	@${CHMOD} g+w ${DOCDIR}/${PROJNAME}
	@for docfile in doc/* verylongdummyname ; do \
		if [ -f $$docfile -o -d $$docfile ] ; then \
			${CP} -R -f $$docfile ${DOCDIR}/${PROJNAME}/ ; \
			echo ${CP} -R -f $$docfile ${DOCDIR}/${PROJNAME}/ ; \
		fi \
	done
	@if ${CHMOD} -R ug+w ${DOCDIR}/${PROJNAME}/ ; then \
		echo "${CHMOD} -R ug+w ${DOCDIR}/${PROJNAME}/" ; \
	fi


refdocinstall:
	@echo " "
	@echo "Installing reference documents"
	@if [ ! -d ${REFDOCDIR} ] ; then \
		${MKDIR} ${REFDOCDIR} ; \
		${CHMOD} g+w ${REFDOCDIR} ; \
		fi
	@${RM} -r -f ${REFDOCDIR}/${PROJNAME}
	@${MKDIR} ${REFDOCDIR}/${PROJNAME}
	@${CHMOD} g+w ${REFDOCDIR}/${PROJNAME}
	@for refdocfile in refdoc/* verylongdummyname ; do \
		if [ -f $$refdocfile -o -d $$refdocfile ] ; then \
			${CP} -R -f $$refdocfile ${REFDOCDIR}/${PROJNAME}/ ; \
			echo ${CP} -R -f $$refdocfile ${REFDOCDIR}/${PROJNAME}/ ; \
		fi \
	done
	@if ${CHMOD} -R ug+w ${REFDOCDIR}/${PROJNAME}/ ; then \
		echo "${CHMOD} -R ug+w ${REFDOCDIR}/${PROJNAME}/" ; \
	fi


xml2doc:
	@echo " "
	@echo "Making documents from xml source"
	@for xmlfile in doc/*.xml refdoc/*.xml verylongdummyname ; do \
	   if [ -f $$xmlfile ] ; then \
		${FSLML} -h $$xmlfile ; \
		${FSLML} -t $$xmlfile ; \
		${FSLML} -m $$xmlfile ; \
		${FSLML} -l $$xmlfile ; \
		echo "${FSLML} $$xmlfile" ; \
	   fi \
	done


tclinstall:
	@echo " "
	@echo "Installing tcl scripts"
	@if [ ! -d ${TCLDIR} ] ; then \
		${MKDIR} ${TCLDIR} ; \
		${CHMOD} g+w ${TCLDIR} ; \
		fi
	@for tclfile in ${TCLFILES} verylongdummyname ; do \
		if [ -f $$tclfile ] ; then \
			${INSTALL} -m 0775 $$tclfile ${TCLDIR}/ ; \
			echo ${INSTALL} -m 0775 $$tclfile ${TCLDIR}/ ; \
		fi \
	done
	( cd ${TCLDIR} ; echo 'auto_mkindex . *.tcl' | ${TCLSH} )


exeinstall:
	@echo " "
	@echo "Installing stable binaries"
	@if [ ! -d ${BINDIR} ] ; then \
		${MKDIR} ${BINDIR} ; \
		${CHMOD} g+w ${BINDIR} ; \
		fi
	@for exefile in ${XFILES} ${SCRIPTS} verylongdummyname ; do \
		if [ -f $$exefile ] ; then \
			${INSTALL} -m 0775 $$exefile ${BINDIR}/ ; \
			echo ${INSTALL} -m 0775 $$exefile ${BINDIR}/ ; \
		fi \
	done


libinstall:
	@echo " "
	@echo "Installing stable library archives"
	@if [ ! -d ${LIBDIR} ] ; then \
		${MKDIR} ${LIBDIR} ; \
		${CHMOD} g+w ${LIBDIR} ; \
		fi
	@for libfile in ${AFILES} verylongdummyname ; do \
		if [ -f $$libfile ] ; then \
			${INSTALL} -m 0664 $$libfile ${LIBDIR} ; \
			echo  ${INSTALL} -m 0664 $$libfile ${LIBDIR} ; \
		fi \
	done


libdevinstall:
	@echo " "
	@echo "Installing development library archives"
	@if [ ! -d ${LIBDIR} ] ; then \
		${MKDIR} ${LIBDIR} ; \
		${CHMOD} g+w ${LIBDIR} ; \
		fi
	@for libfile in ${AFILES} verylongdummyname ; do \
		if [ -f $$libfile ] ; then \
		   libdev=`dirname $$libfile`/`basename $$libfile .a`_dev.a ; export libdev ; \
		   ${INSTALL} -m 0664 $$libfile ${LIBDIR}/$$libdev ; \
		   echo  ${INSTALL} -m 0664 $$libfile ${LIBDIR}/$$libdev ; \
		fi \
	done


hdrinstall:
	@echo " "
	@echo "Installing stable header files"
	@if [ ! -d ${INCDIR} ] ; then \
		${MKDIR} ${INCDIR} ; \
		${CHMOD} g+w ${INCDIR} ; \
		fi
	@if [ ! -d ${INCDIR}/${PROJNAME} ] ; then \
		${MKDIR} ${INCDIR}/${PROJNAME} ; \
		${CHMOD} g+w ${INCDIR}/${PROJNAME} ; \
		fi
	@for hdrfile in ${HFILES} verylongdummyname ; do \
		if [ -f $$hdrfile ] ; then \
			${INSTALL} -m 0664 $$hdrfile ${INCDIR}/${PROJNAME}/ ; \
			echo ${INSTALL} -m 0664 $$hdrfile ${INCDIR}/${PROJNAME}/ ; \
		fi \
	done


hdrdevinstall:
	@echo " "
	@echo "Installing development header files"
	@if [ ! -d ${INCDIR} ] ; then \
		${MKDIR} ${INCDIR} ; \
		${CHMOD} g+w ${INCDIR} ; \
		fi
	@if [ ! -d ${INCDIR}/${PROJNAME}_dev ] ; then \
		${MKDIR} ${INCDIR}/${PROJNAME}_dev ; \
		${CHMOD} g+w ${INCDIR}/${PROJNAME}_dev ; \
		fi
	@for hdrfile in ${HFILES} verylongdummyname ; do \
		if [ -f $$hdrfile ] ; then \
			${INSTALL} -m 0664 $$hdrfile ${INCDIR}/${PROJNAME}_dev/ ; \
			echo ${INSTALL} -m 0664 $$hdrfile ${INCDIR}/${PROJNAME}_dev/ ;\
		fi \
	done


srcinstall:
	@echo " "
	@echo "Installing stable sources"
	@if [ ! -d ${STABLESRCDIR} ] ; then \
		${MKDIR} ${STABLESRCDIR} ; \
		${CHMOD} g+w ${STABLESRCDIR} ; \
		fi
	@if [ ! -d ${STABLESRCDIR}/${PROJNAME} ] ; then \
		${MKDIR} ${STABLESRCDIR}/${PROJNAME} ; \
		${CHMOD} g+w ${STABLESRCDIR}/${PROJNAME} ; \
	else \
		${RM} -r -f ${STABLESRCDIR}/${PROJNAME}~ ; \
		${CP} -R -f ${STABLESRCDIR}/${PROJNAME} ${STABLESRCDIR}/${PROJNAME}~ ; \
		echo ${CP} -R -f ${STABLESRCDIR}/${PROJNAME} ${STABLESRCDIR}/${PROJNAME}~ ; \
		if ${CHMOD} -R ug+w ${STABLESRCDIR}/${PROJNAME}~ ; then \
			echo "${CHMOD} -R ug+w ${STABLESRCDIR}/${PROJNAME}~" ; \
		fi \
	fi
	@if ${CP} ./Makefile ${STABLESRCDIR}/${PROJNAME}/Makefile ; then \
		${RM} -r -f ${STABLESRCDIR}/${PROJNAME} ; \
		${CP} -R -f . ${STABLESRCDIR}/${PROJNAME} ; \
		echo ${CP} -R -f . ${STABLESRCDIR}/${PROJNAME} ; \
	fi
	@if ${CHMOD} -R ug+w ${STABLESRCDIR}/${PROJNAME} ; then \
		echo "${CHMOD} -R ug+w ${STABLESRCDIR}/${PROJNAME}" ; \
	fi
	( cd ${STABLESRCDIR}/${PROJNAME} ; ${MAKE} distclean )


showlibs:
	@echo ${LIBS}

depend: 
	${RM} -f depend.mk
	${MAKE} depend.mk


depend.mk:
	@echo Building dependency file depend.mk
	@for srcfile in *.c* verylongdummyname ; do \
		if [ -f $$srcfile ] ; then \
			${CC} -I${INCDIR} ${USRINCFLAGS} ${DEPENDFLAGS} $$srcfile >> depend.mk ; \
		fi \
	done


include depend.mk










