/*
  hfunc - Functions to manipulate AVW header files
  This has been superceeded by a hfunc library but is still used
  in several older programs

  Stuart Clare, FMRIB Physics Group

  Copyright (C) 1995-1999 University of Oxford
*/

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#include"hfunc.h"
#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include"dbh.h"
#include"hfunc.h"
#define DT_DEFAULT -32767

/* hmake creates an analyse header file

	x,y,z,v: image set dimensions

	t: data type in form
		DT_UNSIGNED_CHAR
		DT_SIGNED_SHORT
		DT_FLOAT
		DT_COMPLEX
		(for others see dbh.h)

	max,min: maximum and minimum data values - use DT_DEFAULT for 
			default settings for particular data type

*/

int hread(char *filename,int* x,int* y,int* z,int* v,int* t,float* vx,float* vy,float* vz,char *orient)
{
  FILE *fp;
  struct dsr header;
  char fname[1024];
		
  xtrt_filepath(filename,fname);
  strcat(fname,".hdr");
  if((fp=fopen(fname,"rb"))==NULL)
    return(-1);

  if(!fread(&header,sizeof(header),1,fp))
    return(-1);

  *x=header.dime.dim[1];
  *y=header.dime.dim[2];
  *z=header.dime.dim[3];
  *v=header.dime.dim[4];
  *t=header.dime.datatype;
  *vx=header.dime.pixdim[1];
  *vy=header.dime.pixdim[2];
  *vz=header.dime.pixdim[3];
  *orient=header.hist.orient;

  fclose(fp);
  return(1);
}
int hread_plus(char *filename,int* x,int* y,int* z,int* v,int* t,float* vx,float* vy,float* vz)
{
  FILE *fp;
  struct dsr header;
  char fname[1024];
		
  xtrt_filepath(filename,fname);
  strcat(fname,".hdr");
  if((fp=fopen(fname,"rb"))==NULL)
    return(-1);

  if(!fread(&header,sizeof(header),1,fp))
    return(-1);

  *x=header.dime.dim[1];
  *y=header.dime.dim[2];
  *z=header.dime.dim[3];
  *v=header.dime.dim[4];
  *t=header.dime.datatype;
  *vx=header.dime.pixdim[1];
  *vy=header.dime.pixdim[2];
  *vz=header.dime.pixdim[3];

  fclose(fp);
  return(1);
}
int hmake(char *filename,int x,int y,int z,int v,int t,int max,int min,float vx,float vy,float vz,char orient)
{
  FILE *fp;
  char dbname[18],fname[1024];
  short *sbuf;
  struct dsr header;

  xtrt_filepath(filename,fname);

  strcat(fname,".hdr");
  if((fp=fopen(fname,"w+b"))==NULL)
    return(-1);	

  xtrt_filename(filename,dbname);
  strcpy(header.hk.db_name,dbname);

  header.hk.sizeof_hdr=348;		/* number of bytes */
  strcpy(header.hk.data_type,"");		
  header.hk.extents=16384;
  header.hk.session_error=0;
  header.hk.regular='r';
  header.hk.hkey_un0=' ';
  header.dime.dim[0]=4;
  header.dime.dim[1]=x;			/* x */
  header.dime.dim[2]=y;			/* y */
  header.dime.dim[3]=z;			/* z */
  header.dime.dim[4]=v;			/* v */
  header.dime.dim[5]=0;
  header.dime.dim[6]=0;
  header.dime.dim[7]=0;
  strcpy(header.dime.vox_units,"mm");
  strcpy(header.dime.cal_units,"");
  header.dime.unused1=0;
  header.dime.datatype=t;			/* see #define statements */
  header.dime.bitpix=16;			/* bits per pixel */
  header.dime.dim_un0=0;
  header.dime.pixdim[0]=0.00;
  header.dime.pixdim[1]=vx;
  header.dime.pixdim[2]=vy;
  header.dime.pixdim[3]=vz;
  header.dime.pixdim[4]=0.00;
  header.dime.pixdim[5]=0.00;
  header.dime.pixdim[6]=0.00;
  header.dime.pixdim[7]=0.00;
  header.dime.vox_offset=0.00;
  header.dime.funused1=1.00;
  header.dime.funused2=0.00;
  header.dime.funused3=0.00;
  header.dime.cal_max=0.00;
  header.dime.cal_min=0.00;
  header.dime.compressed=0;
  header.dime.verified=0;
  switch(t){	
  case(DT_UNSIGNED_CHAR):
    header.dime.datatype=DT_UNSIGNED_CHAR;
    header.dime.bitpix=8;
    if(max==DT_DEFAULT)header.dime.glmax=255;
    else header.dime.glmax=max;
    if(min==DT_DEFAULT)header.dime.glmin=0;
    else header.dime.glmax=min;
    break;
  case(DT_SIGNED_SHORT):
    header.dime.datatype=DT_SIGNED_SHORT;
    header.dime.bitpix=16;
    if(max==DT_DEFAULT)header.dime.glmax=32767;
    else header.dime.glmax=max;
    if(min==DT_DEFAULT)header.dime.glmin=-32767;
    else header.dime.glmin=min;
    break;
  case(DT_FLOAT):
    header.dime.datatype=DT_FLOAT;
    header.dime.bitpix=32;
    if(max==DT_DEFAULT)header.dime.glmax=10;
    else header.dime.glmax=max;
    if(min==DT_DEFAULT)header.dime.glmin=-10;
    else header.dime.glmin=min;
    break;
  case(DT_COMPLEX):
    header.dime.datatype=DT_COMPLEX;
    header.dime.bitpix=64;
    if(max==DT_DEFAULT)header.dime.glmax=32767;
    else header.dime.glmax=max;
    if(min==DT_DEFAULT)header.dime.glmin=-32767;
    else header.dime.glmin=min;
    break;
  }
  strcpy(header.hist.descrip,"Externally created header");
  strcpy(header.hist.aux_file,"");
  header.hist.orient=orient;
  sbuf=(short *)calloc(5,sizeof(short));
  memcpy((void *)header.hist.originator,(void *)sbuf,5*sizeof(short));
  free(sbuf);
  strcpy(header.hist.generated,"");
  strcpy(header.hist.scannum,"");
  strcpy(header.hist.patient_id,"");
  strcpy(header.hist.exp_date,"");
  strcpy(header.hist.exp_time,"");
  strcpy(header.hist.hist_un0,"");
  header.hist.views=0;
  header.hist.vols_added=0;
  header.hist.start_field=0;
  header.hist.field_skip=0;
  header.hist.omax=0;
  header.hist.omin=0;
  header.hist.smax=0;
  header.hist.smin=0;

  fseek(fp,0,0);
  if(fwrite(&header,sizeof(header),1,fp)!=1)
    return(-1);
  fclose(fp);	
  return(1);
}
int hmake_plus(char *filename,int x,int y,int z,int v,int t,int max,int min,float vx,float vy,float vz)
{
  FILE *fp;
  char dbname[18],fname[1024];
  struct dsr header;
  short *sbuf;

  xtrt_filepath(filename,fname);

  strcat(fname,".hdr");
  if((fp=fopen(fname,"w+b"))==NULL)
    return(-1);	

  xtrt_filename(filename,dbname);
  strcpy(header.hk.db_name,dbname);

  header.hk.sizeof_hdr=348;		/* number of bytes */
  strcpy(header.hk.data_type,"");		
  header.hk.extents=16384;
  header.hk.session_error=0;
  header.hk.regular='r';
  header.hk.hkey_un0=' ';
  header.dime.dim[0]=4;
  header.dime.dim[1]=x;			/* x */
  header.dime.dim[2]=y;			/* y */
  header.dime.dim[3]=z;			/* z */
  header.dime.dim[4]=v;			/* v */
  header.dime.dim[5]=0;
  header.dime.dim[6]=0;
  header.dime.dim[7]=0;
  strcpy(header.dime.vox_units,"mm");
  strcpy(header.dime.cal_units,"");
  header.dime.unused1=0;
  header.dime.datatype=t;			/* see #define statements */
  header.dime.bitpix=16;			/* bits per pixel */
  header.dime.dim_un0=0;
  header.dime.pixdim[0]=0.00;
  header.dime.pixdim[1]=vx;
  header.dime.pixdim[2]=vy;
  header.dime.pixdim[3]=vz;
  header.dime.pixdim[4]=0.00;
  header.dime.pixdim[5]=0.00;
  header.dime.pixdim[6]=0.00;
  header.dime.pixdim[7]=0.00;
  header.dime.vox_offset=0.00;
  header.dime.funused1=1.00;
  header.dime.funused2=0.00;
  header.dime.funused3=0.00;
  header.dime.cal_max=0.00;
  header.dime.cal_min=0.00;
  header.dime.compressed=0;
  header.dime.verified=0;
  switch(t){	
  case(DT_UNSIGNED_CHAR):
    header.dime.datatype=DT_UNSIGNED_CHAR;
    header.dime.bitpix=8;
    if(max==DT_DEFAULT)header.dime.glmax=255;
    else header.dime.glmax=max;
    if(min==DT_DEFAULT)header.dime.glmin=0;
    else header.dime.glmax=min;
    break;
  case(DT_SIGNED_SHORT):
    header.dime.datatype=DT_SIGNED_SHORT;
    header.dime.bitpix=16;
    if(max==DT_DEFAULT)header.dime.glmax=32767;
    else header.dime.glmax=max;
    if(min==DT_DEFAULT)header.dime.glmin=-32767;
    else header.dime.glmin=min;
    break;
  case(DT_FLOAT):
    header.dime.datatype=DT_FLOAT;
    header.dime.bitpix=32;
    if(max==DT_DEFAULT)header.dime.glmax=10;
    else header.dime.glmax=max;
    if(min==DT_DEFAULT)header.dime.glmin=-10;
    else header.dime.glmin=min;
    break;
  case(DT_COMPLEX):
    header.dime.datatype=DT_COMPLEX;
    header.dime.bitpix=64;
    if(max==DT_DEFAULT)header.dime.glmax=32767;
    else header.dime.glmax=max;
    if(min==DT_DEFAULT)header.dime.glmin=-32767;
    else header.dime.glmin=min;
    break;
  }
  strcpy(header.hist.descrip,"Externally created header");
  strcpy(header.hist.aux_file,"");
  header.hist.orient=0;
  sbuf=(short *)calloc(5,sizeof(short));
  memcpy((void *)header.hist.originator,(void *)sbuf,5*sizeof(short));
  free(sbuf);
  strcpy(header.hist.generated,"");
  strcpy(header.hist.scannum,"");
  strcpy(header.hist.patient_id,"");
  strcpy(header.hist.exp_date,"");
  strcpy(header.hist.exp_time,"");
  strcpy(header.hist.hist_un0,"");
  header.hist.views=0;
  header.hist.vols_added=0;
  header.hist.start_field=0;
  header.hist.field_skip=0;
  header.hist.omax=0;
  header.hist.omin=0;
  header.hist.smax=0;
  header.hist.smin=0;

  fseek(fp,0,0);
  if(fwrite(&header,sizeof(header),1,fp)!=1)
    return(-1);
  fclose(fp);	
  return(1);
}
void xtrt_filepath(char *in,char *out)
{
  int len;	
  len=strlen(in);
  if(!strcmp(in+len-4,".img"))len-=4;
  if(!strcmp(in+len-4,".hdr"))len-=4;
  strcpy(out,in);
  out[len]='\0';
}
void xtrt_filename(char *in,char *out)
{
  char tmp[128];
  int len;

  if(strrchr(in,'/')==NULL)strcpy(tmp,in);
  else strcpy(tmp,strrchr(in,'/')+1);
  len=strlen(tmp);
  if(!strcmp(tmp+len-4,".img"))len-=4;
  if(!strcmp(tmp+len-4,".hdr"))len-=4;
  strncpy(out,tmp,17);
  if(len>17)len=17;
  out[len]='\0';  
}
