#

#{{{ copyright and setup

#   FEAT - FMRIB's Easy Analysis Tool for FMRI
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

source ${FSLDIR}/tcl/feat3_proc.tcl

#}}}
#{{{ proc feat

proc feat3 { w } {
    global fmri
    global PXHOME
    global FSLDIR
    global USER
    global PROCID
    global VERSION
    global INMEDX
 
    #{{{ setup main window

toplevel $w

set fmri(version) $VERSION

set fmri(feat_filename) /tmp/feat_model_${USER}_${PROCID}.fsf

wm title      $w "FEAT - FMRIB's Easy Analysis Tool - v$fmri(version)"
wm iconname   $w "FEAT"
wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm

tixBalloon $w.bhelp -initwait 3000 -options { background #bfdfff }

frame $w.f

#}}}
    #{{{ TR, npts, ndelete

frame $w.f.tr_and_npts

#{{{ TR

set fmri(tr) 3.0
tixControl $w.f.tr -label " TR" -variable fmri(tr) \
	-step 0.25 -min 0.001 -selectmode immediate -options { entry.width 4 }
$w.bhelp bind $w.f.tr -msg "The time (in seconds) between scanning successive FMRI volumes."

#}}}
#{{{ npts

set fmri(npts) 0

if { $INMEDX } {
    if { [ current_group 0 ] } {
	MxGetCurrentPage Group
	MxGetPagesFromGroup $Group GPages
	set fmri(npts) [llength $GPages]
    }
} else {
    set fmri(npts) 100
}

tixControl $w.f.npts -label "   Total volumes" -variable fmri(npts) \
	-step 1 -min 0 -selectmode immediate -options { entry.width 4 }
$w.bhelp bind $w.f.npts -msg "The number of FMRI volumes in the time series, including any initial
volumes that you wish to delete."

#}}}
#{{{ ndelete

set fmri(ndelete) 0

tixControl $w.f.ndelete -label "   Delete volumes" -variable fmri(ndelete) \
	-step 1 -min 0 -selectmode immediate -options { entry.width 3 }

$w.bhelp bind $w.f.ndelete -msg "The number of initial FMRI volumes to delete before any further
processing. These should be the volumes that are not wanted because
steady-state imaging has not yet been reached - typically one or two
volumes. These volumes are deleted as soon as FEAT is started, so all
4D data sets produced by FEAT will not contain the deleted volumes.

Note that \"Delete volumes\" should not be used to correct for the
time lag between stimulation and the measured response - this is
corrected for in the design matrix by convolving the input stimulation
waveform with a blurring-and-delaying haemodynamic response function.

Most importantly, remember when setting up the design matrix, that the
timings in the design matrix start at t=0 seconds, and this
corresponds to the start of the first image taken after the deleted
scans. In other words, the design matrix starts AFTER the \"deleted
scans\" have been deleted."

#}}}

pack $w.f.tr $w.f.npts $w.f.ndelete -in $w.f.tr_and_npts -side left

#}}}
    #{{{ paradigm

tixLabelFrame $w.f.paradigm -label "Paradigm"
set fmri(lfparadigm) [$w.f.paradigm subwidget frame]

#{{{ choose paradigm type

frame $w.f.paradigm_type

set fmri(paradigm_type) 1	

radiobutton $w.f.paradigm_type.jab -text "ABAB..." -value 1 -variable fmri(paradigm_type) -command "feat3:updateparadigm $w 0 1"
radiobutton $w.f.paradigm_type.jabac -text "ABACABAC..." -value 2 -variable fmri(paradigm_type) -command "feat3:updateparadigm $w 0 1"
radiobutton $w.f.paradigm_type.ud -text "General linear model" -value 3 -variable fmri(paradigm_type) -command "feat3:updateparadigm $w 0 1"
radiobutton $w.f.paradigm_type.er -text "Model-free" -value 4 -variable fmri(paradigm_type) -command "feat3:updateparadigm $w 0 1"

pack $w.f.paradigm_type.jab $w.f.paradigm_type.jabac $w.f.paradigm_type.ud $w.f.paradigm_type.er -in $w.f.paradigm_type -side top -padx 5 -side left

#}}}
#{{{ frame ab

frame $w.f.ab

set fmri(a_count) 30
tixControl $w.f.a_count -label "A (rest) period" \
	-variable fmri(a_count) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

set fmri(b_count) 30
tixControl $w.f.b_count -label "B period" \
	-variable fmri(b_count) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

set fmri(c_count) 30
tixControl $w.f.c_count -label "C period" \
	-variable fmri(c_count) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

button $w.f.view -command "feat3:updateparadigm $w 1 0" -text "Process"

pack $w.f.a_count $w.f.b_count $w.f.view -in $w.f.ab -side left -padx 5 -pady 3 -anchor n

$w.bhelp bind $w.f.a_count -msg "The A period (seconds) is normally the rest period, i.e., no
stimulation was applied during this period."

$w.bhelp bind $w.f.b_count -msg "The B period (seconds) is normally the activation period, i.e.,
stimulation was applied during this period."

$w.bhelp bind $w.f.c_count -msg "The C period (seconds) is normally the period associated with a second
type of activation, i.e., stimulation type 2 was applied during this
period."

#}}}
#{{{ general linear model

frame $w.f.ud

#{{{ setup model

set fmri(w_model) 0

set fmri(evs) 1
tixControl $w.f.evs -label "EVs " -variable fmri(evs) -step 1 -min 1 -command "feat3:setup_model_evs $w"

$w.bhelp bind $w.f.evs -msg "The basic number of explanatory variables in the design matrix; this
means the number of different effects that you wish to model - one for
each modelled stimulation type, and one for each modelled confound."

button $w.f.model -text "Edit model" -command "feat3:setup_model $w"

#}}}
#{{{ high pass

set fmri(paradigm_hp) 100
tixControl $w.f.paradigm_hp -label "High pass filter cutoff" \
	-variable fmri(paradigm_hp) -step 5 -min 1 -selectmode immediate -options { entry.width 6 }

$w.bhelp bind $w.f.paradigm_hp -msg "The high pass frequency cutoff point (seconds), that is, the longest
temporal period that you will allow. A sensible setting in the case of
an ABAB or ABACABAC type block design is 1.5 * the (A+B) or (A+B+A+C)
total cycle time."

#}}}

pack $w.f.evs $w.f.model $w.f.paradigm_hp -in $w.f.ud -side left -padx 5 -pady 3

feat3:setup_model_init $w

#}}}
#{{{ frame model-free

frame $w.f.mf

set fmri(mf_type) 1	
radiobutton $w.f.mf_cyclic    -text "Cyclic"       -value 1 -variable fmri(mf_type) -command "feat3:updatemf $w 1"
radiobutton $w.f.mf_triggered -text "Triggered   " -value 2 -variable fmri(mf_type) -command "feat3:updatemf $w 1"
$w.bhelp bind $w.f.mf_cyclic -msg "Select whether to use a cyclic model-free analysis (normally for
\"block-like\" experiments) or triggered analysis (normally for
irregularly-spaced event-related experiments). Regular event-related
experiments could be analysed either way."
$w.bhelp bind $w.f.mf_triggered -msg "Select whether to use a cyclic model-free analysis (normally for
\"block-like\" experiments) or triggered analysis (normally for
irregularly-spaced event-related experiments). Regular event-related
experiments could be analysed either way."

set fmri(mf_cycle) 20
tixControl $w.f.mf_cycle -label "Volumes for complete cycle" \
	-variable fmri(mf_cycle) -step 1 -min 1 -selectmode immediate -options { entry.width 4 }
$w.bhelp bind $w.f.mf_cycle -msg "The period (volumes) of each stimulation cycle (e.g., for an ABAB
design, this is A+B)."

tixControl $w.f.mf_vols -label "Volumes per event" \
	-variable fmri(mf_cycle) -step 1 -min 1 -selectmode immediate -options { entry.width 4 }
$w.bhelp bind $w.f.mf_vols -msg "The number of volumes to include in analysis after each trigger in the
event file."

set fmri(mf_file) "~"
tixFileEntry $w.f.mf_file -label " File of events" \
	-variable fmri(mf_file) -options { entry.width 15 }
$w.f.mf_file filedialog subwidget fsbox config -directory "~" -pattern "*"
$w.bhelp bind $w.f.mf_file -msg "The event file - this should be a list of events (in volumes counts,
with first volume referred to as \"0\")."

pack $w.f.mf_cyclic $w.f.mf_triggered $w.f.mf_cycle -in $w.f.mf -side left -padx 5 -pady 3

#}}}

pack $w.f.paradigm_type $w.f.ab -in $fmri(lfparadigm) -anchor w

$w.bhelp bind $w.f.paradigm -msg "Select the type of experiment to be analysed:

\"ABAB...\" means alternating A (rest) and B (activation) groups.

\"ABACABAC...\" is similar, but the activation groups alternate
between two different types - B and C. With either of the above, set
the number of seconds in each period, then press \"Process\" to
convert to the \"General design matrix\" format \"General desgin
matrix\" allows full control over the linear model that is fit to the
data, including event-related experiments.

\"Model-free\" uses the ANOVA model-free analysis to process data for
which the expected response to the stimulus is not well-known in
advance."

$w.bhelp bind $w.f.paradigm_type -msg "Select the type of experiment to be analysed:

\"ABAB...\" means alternating A (rest) and B (activation) groups.

\"ABACABAC...\" is similar, but the activation groups alternate
between two different types - B and C. With either of the above, set
the number of seconds in each period, then press \"Process\" to
convert to the \"General design matrix\" format \"General desgin
matrix\" allows full control over the linear model that is fit to the
data, including event-related experiments.

\"Model-free\" uses the ANOVA model-free analysis to process data for
which the expected response to the stimulus is not well-known in
advance."

#}}}
    #{{{ Advanced options

collapsible frame $w.f.opts -title "Advanced options"

#{{{ option menus

frame $w.f.opts.menus

set fmri(type) 2
tixOptionMenu $w.f.opts.menus.type -variable fmri(type) -command "feat3:updatetype $w 1"
$w.f.opts.menus.type add command 2 -label "Best"
$w.f.opts.menus.type add command 1 -label "Fastest"
$w.bhelp bind $w.f.opts.menus.type -msg "Choose whether you want FEAT to be as fast as possible or give the
best possible results."

global fmri(analysis)
tixOptionMenu $w.f.opts.menus.analysis -variable fmri(analysis)
$w.f.opts.menus.analysis add command 7 -label "Full analysis"
$w.f.opts.menus.analysis add command 1 -label "Pre-Stats"
$w.f.opts.menus.analysis add command 3 -label "Pre-Stats + Stats"
$w.f.opts.menus.analysis add command 2 -label "                     Stats"
$w.f.opts.menus.analysis add command 6 -label "                     Stats + Contrasts, Thresholding, Rendering"
$w.f.opts.menus.analysis add command 4 -label "                                  Contrasts, Thresholding, Rendering"
$w.bhelp bind $w.f.opts.menus.analysis -msg "You can run a full analysis: Pre-Stats; Stats; Contrasts (when GLM is
selected), Thresholding and Rendering, or a (sensible) subset of these
options. If you run Constrasts, Thresholding and Rendering only, you
will need to select a FEAT directory; the stats results in this will
be used to rerun these final analysis stages."

pack $w.f.opts.menus.type $w.f.opts.menus.analysis -in $w.f.opts.menus -side left -anchor n

#}}}

tixNoteBook $w.f.opts.nb
$w.f.opts.nb add misc -label "Misc"
$w.f.opts.nb add filtering -label "Pre-Stats"
$w.f.opts.nb add stats -label "Stats"
$w.f.opts.nb add poststats -label "Contrasts, Thresholding, Rendering"

#{{{ Misc

set fmri(miscf) [ $w.f.opts.nb subwidget misc ]

#{{{ multiple analyses

frame $w.f.multiple

set fmri(multiple) 0
tixControl $w.f.multiple.number -label "File-based analyses " -padx 2 \
	-variable fmri(multiple) -step 1 -min 0 -selectmode immediate \
	-command "feat3:updateselect $w 1" -options { entry.width 3 }

button $w.f.multiple.setup -text "Select 4D data" -command "feat3:multiple_select $w"

pack $w.f.multiple.number -in $w.f.multiple -side left -padx 5

$w.bhelp bind $w.f.multiple -msg "The number of experiments that you wish to analyse. Normally you will
just be analysing a single data set; open a folder and load up the
group page of raw FMRI data before pressing Apply or OK at the bottom
of FEAT. If you wish to analyse several data sets in one go, increase
the \"Number of analyses\". The data sets must already be 4D Analyze
files saved to disk. Enter these by pressing the button that appears
when you increase the \"Number of analyses\". Of course, all of the
analyses will use the same FEAT settings."

#}}}
#{{{ delay

set fmri(delay) 0

tixControl $w.f.delay -label " Delay before starting (hours)" \
	-variable fmri(delay) -step 1 -min 0 -selectmode immediate -padx 2

$w.bhelp bind $w.f.delay -msg "If you are using FEAT to carry out multiple analyses, you should
probably do this overnight, to reduce the load on the computer
system. You can use this delay feature to tell FEAT how long to wait
before starting the analyses - for example, so that FEAT does not
start before 7pm."

#}}}
#{{{ save options

frame $w.f.save

label $w.f.save.prep_label -text "  Save filtered data"
set fmri(saveprep_yn) 1
checkbutton $w.f.save.prep_yn -variable fmri(saveprep_yn)

label $w.f.save.folder_label -text "Save folder"
set fmri(savefolder_yn) 0
checkbutton $w.f.save.folder_yn -variable fmri(savefolder_yn)

pack $w.f.save.prep_label $w.f.save.prep_yn $w.f.save.folder_label $w.f.save.folder_yn -in $w.f.save -side left -padx 0

$w.bhelp bind $w.f.save -msg "FEAT will save its results in a directory named \"something.feat\",
where the filename is derived from the file and directory name of the
input dataset (if possible). Otherwise, it will save results in your
home directory. FEAT then saves all results (mostly Analyze format
images) and setup files in this directory. There are two additional
options:

\"Save filtered data\" saves the 4D FMRI data into an Analyze file
after all pre-stats filtering.

\"Save folder\" saves the MEDx session into a folder within the FEAT
directory, in addition to the Analyze files. This is wasteful of disk
space, but makes it easy for you to go back and play with the data
later."

#}}}
#{{{ balloon help

frame $w.f.help

label $w.help_label -text "  Balloon help"
set fmri(help_yn) 1
checkbutton $w.help_yn -variable fmri(help_yn) -command "feat3:updatehelp $w"

pack $w.help_label $w.help_yn -in $w.f.help -side left -padx 0

$w.bhelp bind $w.help_label -msg "And don't expect this message to appear whilst you've turned this
option off!"

#}}}
#{{{ brain threshold

set fmri(brain_thresh) 10
tixControl $w.f.brain_thresh -label "  Brain/background threshold, % " \
	-variable fmri(brain_thresh) -step 1 -min 0 -max 100 -selectmode immediate

$w.bhelp bind $w.f.brain_thresh -msg "This is automatically calculated, as a % of the maximum input image
intensity. It is used in motion correction, intensity normalisation
and brain mask generation. You may need to change this in exceptional
circumstances, for example, if there is strong \"ghosting\" in the FMRI
images."

#}}}

pack $w.f.multiple $w.f.delay $w.f.save $w.f.help $w.f.brain_thresh -in $fmri(miscf) -anchor w -side top

#}}}
#{{{ Pre-statistics processing

set fmri(filteringf) [ $w.f.opts.nb subwidget filtering ]

set fmri(filtering_yn) 1

#{{{ motion correction

frame $w.f.mc
set fmri(mcf) $w.f.mc

label $w.f.mc.label -text "Motion correction: "

global fmri(mc)
tixOptionMenu $w.f.mc.menu -label "Motion correction: " -variable fmri(mc) -command "feat3:updatemc $w"
$w.f.mc.menu add command 0 -label "None"
$w.f.mc.menu add command 1 -label "Auto-decide: AIR 2D or 3D"
$w.f.mc.menu add command 2 -label "AIR 2D"
$w.f.mc.menu add command 3 -label "AIR 3D"
$w.f.mc.menu add command 4 -label "SPM"
set fmri(mc) 1

pack $w.f.mc.menu -in $fmri(mcf) -side top -padx 5 -side left

$w.bhelp bind $w.f.mc -msg "You will normally want to apply motion correction; this attempts to
remove the effect of subject head motion during the experiment. You
can choose between the \"AIR\" and \"SPM\" methods. They often give
somewhat different results, but on average neither is clearly
\"better\". If using \"AIR\", it is simplest to select \"Auto-decide:
AIR 2D or 3D\"; FEAT then chooses to do 2D processing if the number of
slices is less than 7, and 3D otherwise. \"2D\" means that no
out-of-slice motion is allowed. If using \"SPM\", only 3D processing
is possible."

#}}}
#{{{ SPM spin history

frame $w.f.sh

label $w.f.sh.label -text "Adjustment for movement"

set fmri(sh_yn) 1
checkbutton $w.f.sh.yn -variable fmri(sh_yn)

pack $w.f.sh.label $w.f.sh.yn -in $w.f.sh -padx 5 -side left

$w.bhelp bind $w.f.sh -msg "It is normally recommended that SPM \"adjustment for movement\" (often
referred to as \"spin history correction\") be applied. This attempts
to remove temporal effects which are correlated with the estimated
motion correction parameters, as these are likely to be motion-induced
rather than stimulation-induced. FEAT automatically sets the correct
type of \"adjustment for movement\"."

#}}}
#{{{ spatial filtering

set fmri(smooth) 5
tixControl $w.f.smooth -label " Spatial smoothing FWHM (mm)" \
	-variable fmri(smooth) -step 1 -min 0 -selectmode immediate

$w.bhelp bind $w.f.smooth -msg "This determines the extent of the spatial smoothing, carried out on
each volume of the FMRI data set separately. This is intended to
reduce noise without reducing valid activation; this is successful as
long as the underlying activation area is larger than the extent of
the smoothing. Thus if you are looking for very small activation areas
then you should maybe reduce smoothing from the default of 5mm, and if
you are looking for larger areas, you can increase it, maybe to 10 or
even 15mm."

#}}}
#{{{ intensity normalization

frame $w.f.norm

label $w.f.norm.label -text "Intensity normalization"

set fmri(norm_yn) 0
checkbutton $w.f.norm.yn -variable fmri(norm_yn)

pack $w.f.norm.label $w.f.norm.yn -in $w.f.norm -side left -padx 3

$w.bhelp bind $w.f.norm -msg "This forces every FMRI volume to have the same mean intensity. For
each volume it calculates the mean intensity and then scales the
intensity across the whole volume so that the global mean becomes a
preset constant. This step is normally discouraged - hence is turned
off by default. When this step is not carried out, the whole 4D data
set is still normalised by a single scaling factor (\"grand mean
scaling\") - each volume is scaled by the same amount. This is so that
inter-subject and inter-session second-level analyses are valid."

#}}}
#{{{ temporal filtering

frame $w.f.temp

label $w.f.temp.label -text " Temporal filtering    "

set fmri(temphp_yn) 1
label $w.f.temp.hplabel -text "Highpass"
checkbutton $w.f.temp.hp_yn -variable fmri(temphp_yn)
set fmri(templp_yn) 0
label $w.f.temp.lplabel -text "Lowpass"
checkbutton $w.f.temp.lp_yn -variable fmri(templp_yn)

pack $w.f.temp.label $w.f.temp.hplabel $w.f.temp.hp_yn $w.f.temp.lplabel $w.f.temp.lp_yn -in $w.f.temp -side top -side left

$w.bhelp bind $w.f.temp -msg "Nonlinear \"highpass\" temporal filtering uses a local fit of a straight
line (Gaussian-weighted within the line to give a smooth response) to
remove low frequency artefacts. This is preferable to standard linear
filtering as it does not introduce new autocorrelations into the data.

Lowpass filtering reduces high frequency noise with Gaussian
smoothing. With the ANOVA software used for \"Model-free\" analysis,
Gaussian smoothing invalidates the statistics, so in this case, by
default, only high pass filtering is applied by FEAT."

#}}}

pack $fmri(mcf) $w.f.smooth $w.f.norm $w.f.temp -in $fmri(filteringf) -anchor w -pady 0

#}}}
#{{{ Statistics

set fmri(statsf) [ $w.f.opts.nb subwidget stats ]

set fmri(stats_yn) 1

label $w.f.statsoptionlabel -text "Will run GLM analysis"

pack $w.f.statsoptionlabel -in $fmri(statsf) -side top -anchor w -padx 10 -pady 10

#}}}
#{{{ Post-Stats

set fmri(lfpoststats) [ $w.f.opts.nb subwidget poststats ]

set fmri(poststats_yn) 1

#{{{ thresholding

tixLabelFrame $w.f.thresh -label "Thresholding"
set fmri(lfthresh) [ $w.f.thresh subwidget frame ]

global fmri(thresh)
tixOptionMenu $w.f.thresh.menu -variable fmri(thresh)
$w.f.thresh.menu add command 0 -label "None"
$w.f.thresh.menu add command 1 -label "Uncorrected"
$w.f.thresh.menu add command 2 -label "Voxel"
$w.f.thresh.menu add command 3 -label "Cluster"

set fmri(prob_thresh) 0.01
set fmri(z_thresh) 2.3

tixControl $w.f.uncorrected -label "Uncorrected voxel P threshold" \
	-variable fmri(prob_thresh) -step 0.01 -min 0 -max 1 -selectmode immediate
tixControl $w.f.voxel -label "Corrected voxel P threshold" \
	-variable fmri(prob_thresh) -step 0.01 -min 0 -max 1 -selectmode immediate
tixControl $w.f.prob_thresh -label " Cluster P threshold" \
	-variable fmri(prob_thresh) -step 0.01 -min 0 -max 1 -selectmode immediate
tixControl $w.f.z_thresh -label "Z threshold" \
	-variable fmri(z_thresh) -step 0.1 -min 0 -selectmode immediate

pack $w.f.thresh.menu -in $fmri(lfthresh) -side top -padx 5 -side left

$w.bhelp bind $w.f.thresh -msg "After carrying out the initial statistical test, the resulting Z
statistic image is then normally thresholded to show which voxels or
clusters of voxels are activated at a particular significance level.

If \"Cluster\" thresholding is selected, a Z statistic threshold is
used to define contiguous clusters.  Then each cluster's estimated
significance level is compared with the cluster probability
threshold. Significant clusters are then used to mask the original Z
statistic image for later production of colour blobs. (This last stage
is not normally done by MEDx - MEDx normally stops at the production
of significant clusters. Note also that the Z statistic values which
survived masking by the clusters are still uncorrected Z statistics.)
This method of thresholding is an alternative to multiple comparison
correction (\"Voxel\"), and is normally more sensitive to
activation. You may well want to increase the \"Cluster creation Z
threshold\" if you have reasonable or high levels of activation.

If \"Voxel\" thresholding is selected, multiple comparison resel
correction is carried out, with thresholding at the chosen level. The
difference between this and Bonferoni multiple comparison correction
is that the number of voxels used in the calculation is reduced from
the original number of brain voxels to the number of \"independent\"
voxels, or \"resels\". Thus this test is less overly-conservative than
Bonferoni correction.

You can also choose to simply threshold the uncorrected Z statistic
values, or apply no thresholding at all."

#}}}
#{{{ rendering

tixLabelFrame $w.f.render -label "Rendering"
set fmri(lfrendering) [ $w.f.render subwidget frame ]

#{{{ Z display min and max

global fmri(zdisplay)
tixOptionMenu $w.f.zmaxmenu -variable fmri(zdisplay)
$w.f.zmaxmenu add command 0 -label "Use actual Z min/max"
$w.f.zmaxmenu add command 1 -label "Use preset Z min/max"

set fmri(zmin) 1
tixControl $w.f.zmin -label "Min" -variable fmri(zmin) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }
set fmri(zmax) 15
tixControl $w.f.zmax -label "Max" -variable fmri(zmax) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }

$w.bhelp bind $w.f.zmaxmenu -msg "The Z statistic range selected for rendering is automatically
calculated by default, to run from red (minimum Z statistic after
thresholding) to yellow (maximum Z statistic). If more than one colour
rendered image is to be produced (i.e., when multiple constrasts are
created) then the overall range of Z values is automatically found
from all of the Z statistic images, for consistent Z statistic
colour-coding.

If multiple analyses are to be carried out, \"Use preset Z min/max\"
should be chosen, and the min/max values set by hand. Again, this
ensures consistency of Z statistic colour-coding - if several
experiments are to be reported side-by-side, colours will refer to the
same Z statistic values in each picture. When using this option, you
should choose a conservatively wide range for the min and max (e.g.,
min=1, max=15), to make sure that you do not carry out unintentional
thresholding via colour rendering."

#}}}
#{{{ render type

tixOptionMenu $w.f.rendertype -variable fmri(rendertype)

$w.f.rendertype add command 0 -label "Solid blobs"
$w.f.rendertype add command 1 -label "Transparent blobs"

set fmri(rendertype) 1

$w.bhelp bind $w.f.rendertype -msg "With \"Solid colours\" you don't see any sign of the background images
within the colour blobs; with \"Transparent colours\" you will see
through the colour blobs to the background intensity"

#}}}

pack $w.f.zmaxmenu $w.f.rendertype -in $fmri(lfrendering) -side left -anchor n -padx 5 -pady 5

#}}}

pack $w.f.thresh $w.f.render -in $fmri(lfpoststats) -side top -anchor w

$w.f.thresh.menu configure -command "feat3:updatepoststats $w"
$w.f.zmaxmenu configure -command "feat3:updatepoststats $w"
set fmri(thresh) 3
set fmri(zdisplay) 0

#$w.bhelp bind $w.poststats_yn -msg "If at least one voxel has survived thresholding, colour rendering is
#carried out, unless you turn rendering off. Significant voxels are
#overlaid in colour on top of one of the original FMRI images."

#}}}
#{{{ COMMENT High resolution image

#frame $w.f.highresf
#label $w.f.highresl -text  " High resolution image "
#entry $w.f.highres -textvariable fmri(highres) -width 35
#pack $w.f.highresl $w.f.highres -in $w.f.highresf -side left
#
#$w.bhelp bind $w.f.highresl -msg "If you paste in a high resolution image name from the page manager, one of the low resolution\nFMRI images will be registered to the high resolution image using FLIRT. The resulting\ntransform will be applied to the Z score image(s) and colour rendering will then be carried\nout using the high resolution Z score image(s) and the high resolution MRI image."

#}}}

pack $w.f.opts.menus $w.f.opts.nb -in $w.f.opts.b -side top -anchor w -pady 5 -padx 5

$w.f.opts.menus.analysis configure -command "feat3:updateanalysis $w"
set fmri(analysis) 7

#}}}

    pack $w.f.tr_and_npts $w.f.paradigm $w.f.opts -in $w.f -anchor w -pady 4
    pack $w.f -in $w -side top -padx 5 -pady 5

    #{{{ Button Frame

frame $w.btns
frame $w.btns.b -relief raised -borderwidth 1
    
button $w.apply -command "feat3:apply $w" -text "Go"

tixFileSelectDialog .savedialog -title "Save FEAT setup" -command "feat3:write $w"
.savedialog subwidget fsbox config -directory "~" -pattern "*.fsf"
button $w.save -command ".savedialog popup" -text "Save"

tixFileSelectDialog .loaddialog -title "Load FEAT setup" -command "feat3:load $w 1"
.loaddialog subwidget fsbox config -directory "~" -pattern "*.fsf"
button $w.load -command ".loaddialog popup" -text "Load"

button $w.cancel    -command "destroy $w" -text "Cancel" -width 5

button $w.help -command "FmribWebHelp file: ${FSLDIR}/doc/feat3/index.html" -text "Help" -width 5

pack $w.btns.b -side bottom -fill x -padx 3 -pady 3
pack $w.apply $w.save $w.load $w.cancel $w.help -in $w.btns.b \
	-side left -expand yes -padx 3 -pady 3 -fill y

pack $w.btns -in $w -side bottom -fill x

#}}}
}

#}}}
#{{{ proc feat3:apply

proc feat3:apply { w } {
    global fmri
    global PXHOME
    global feat_files
    global USER
    global PROCID
    global RENICE
    global INMEDX

    #{{{ in MEDx?

if { ! $INMEDX } {
    MxPause "Don't be silly - you're not running MEDx!"
    return 1
}

#}}}
    #{{{ test for ABAB or ABAC

if { $fmri(paradigm_type) < 3 } {
    MxPause "Please press \"Process\" to convert simple paradigm to general design matrix"
    return
}

#}}}
    #{{{ update variables

foreach v { tr npts ndelete paradigm_hp mf_cycle brain_thresh smooth uncorrected voxel prob_thresh z_thresh zmin zmax } {
    $w.f.$v update
}

#}}}
    #{{{ write model

if { [ feat3:write $w $fmri(feat_filename) ] } {
    return 1
}

#}}}
    #{{{ wait if required

if { $fmri(delay) > 0.003 } {
    ScriptUpdate "FEAT will begin processing in $fmri(delay) hours.\n\n" 
    after [ expr int( $fmri(delay) * 60.0 * 60.0 * 1000.0 ) ]
    CancelScriptUpdate
}

#}}}

    if { $fmri(multiple) > 0 } {
	#{{{ run feat3_proc for each filename

if { $RENICE && $fmri(multiple) > 1 } {
    set result [catch {eval exec /usr/local/bin/nicemedx 18} ErrMsg]
}

set i 1
while { $i <= $fmri(multiple) } {

    MxNewFolder Folder Window

    if { $fmri(analysis) != 4 } {
	MxOpenImage $Folder $feat_files($i) Group
	if { [ current_group 0 ] } {
	    feat3_proc [ file rootname $fmri(feat_filename) ] 1 dummyfeatdir
	}
    } else {
	feat3_proc [ file rootname $fmri(feat_filename) ] 1 $feat_files($i)
    }
    
    if { $fmri(multiple) > 1 } {
	after 30000
	MxCloseWindow $Window
    }

    incr i 1
}

#}}}
    } else {
	#{{{ run feat on current group

    if { [ current_group 1 ] } {
	feat3_proc [ file rootname $fmri(feat_filename) ] 1 dummyfeatdir
    }

#}}}
    }

    update idletasks
}

#}}}
#{{{ updates

#{{{ feat3:updatetype

proc feat3:updatetype { w force dummy } {
    global fmri

    if { $force } {
	if { $fmri(type) == 1 } {
	    set fmri(mc) 0
	} elseif { $fmri(type) == 2 } {
	    set fmri(mc) 1
	}
    }
}

#}}}
#{{{ feat3:updateparadigm

proc feat3:updateparadigm { w process force } {
    global fmri

    if { [ info exists fmri(w_model) ] } {
	if { [ winfo exists $fmri(w_model) ] } {
	    destroy $fmri(w_model)
	}
    }

    if { $process } {
	feat3:setup_model_vars_simple $w
	set fmri(paradigm_type) 3
	if { [ feat3:setup_model_preview $w ] } {
	    return 1
	}
    }

    if { $fmri(paradigm_type) == 1 } {
	set fmri(a_count) 30
	set fmri(b_count) 30
	pack forget $w.f.mf
	pack forget $w.f.ud
	pack $w.f.ab -in $fmri(lfparadigm) -side left
	pack forget $w.f.c_count
	$w.f.statsoptionlabel configure -text "Will run GLM analysis"
	if { $force } {
	    set fmri(templp_yn) 0
	}
    } elseif { $fmri(paradigm_type) == 2 } {
	set fmri(a_count) 30
	set fmri(b_count) 30
	set fmri(c_count) 30
	pack forget $w.f.mf
	pack forget $w.f.ud
	pack $w.f.ab -in $fmri(lfparadigm) -side left
	pack $w.f.c_count -in $w.f.ab -after $w.f.b_count -side left -padx 5 -pady 3 -anchor n
	$w.f.statsoptionlabel configure -text "Will run GLM analysis"
	if { $force } {
	    set fmri(templp_yn) 0
	}
    } elseif { $fmri(paradigm_type) == 3 } {
	pack forget $w.f.ab
	pack forget $w.f.mf
	pack $w.f.ud -in $fmri(lfparadigm) -side left
	if { ! $process } {
	    feat3:setup_model $w
	}
	$w.f.statsoptionlabel configure -text "Will run GLM analysis"
	if { $force } {
	    set fmri(templp_yn) 0
	}
    } else {
	set fmri(a_count) 20
	set fmri(b_count) 0
	pack forget $w.f.ab
	pack forget $w.f.ud
	pack $w.f.mf -in $fmri(lfparadigm) -side left
	$w.f.statsoptionlabel configure -text "Will run Model-free analysis"
	if { $force } {
	    set fmri(templp_yn) 0
	}
    }
}

#}}}
#{{{ feat3:updatemf

proc feat3:updatemf { w force } {
    global fmri

    if { $fmri(mf_type) == 1 } {
	pack forget $w.f.mf_vols
	pack forget $w.f.mf_file
	pack $w.f.mf_cycle -in $w.f.mf -side left
    } else {
	pack forget $w.f.mf_cycle
	pack $w.f.mf_vols $w.f.mf_file -in $w.f.mf -side left
	if { $force } {
	    $w.f.mf_file filedialog popup
	}
    }
}

#}}}
#{{{ feat3:updateanalysis

proc feat3:updateanalysis { w dummy } {
    global fmri

    set fmri(filtering_yn) [ expr   $fmri(analysis)       % 2 == 1 ]
    set fmri(stats_yn)     [ expr ( $fmri(analysis) / 2 ) % 2 == 1 ]
    set fmri(poststats_yn) [ expr ( $fmri(analysis) / 4 ) % 2 == 1 ]

    feat3:updatefiltering $w
    feat3:updatestats $w
    feat3:updatepoststats $w 0

    $w.f.opts.nb raise misc
}

#}}}
#{{{ feat3:updateselect

proc feat3:updateselect { w force dummy } {
    global fmri

    if { $fmri(analysis) != 4 } {
	$w.f.multiple.setup configure -text "Select 4D data"
	$w.f.multiple.number configure -min 0
    } else {
	$w.f.multiple.number configure -min 1
	if { $fmri(multiple) == 0 } {
	    set fmri(multiple) 1
	}
	if { $fmri(multiple) == 1 } {
	    $w.f.multiple.setup configure -text "Select FEAT directory"
	} else {
	    $w.f.multiple.setup configure -text "Select FEAT directories"
	}
    }

    if { $fmri(multiple) == 0 } {
	pack forget $w.f.multiple.setup
    } else {
	pack $w.f.multiple.setup -in $w.f.multiple -side left -padx 5
    }

    if { $force && $fmri(analysis) != 4 && $fmri(multiple) > 1 } {
	set fmri(delay) 10
    }
}

#}}}
#{{{ feat3:updatehelp

proc feat3:updatehelp { w } {
    global fmri

    if { $fmri(help_yn) == 1 } {
	$w.bhelp configure -state balloon
    } else {
	$w.bhelp configure -state none
    }
}

#}}}
#{{{ feat3:updatefiltering

proc feat3:updatefiltering { w } {
    global fmri

    if { !$fmri(filtering_yn) } {
	set fmri(saveprep_yn) 0
	$w.f.opts.nb pageconfigure filtering -state disabled
    }  else {
	$w.f.opts.nb pageconfigure filtering -state normal
    }
}

#}}}
#{{{ feat3:updatemc

proc feat3:updatemc { w dummy } {
    global fmri

    if { $fmri(mc) != 4 } {
	pack forget $w.f.sh
    } else {
	pack $w.f.sh -in $fmri(mcf) -side left
    }
}

#}}}
#{{{ feat3:updatestats

proc feat3:updatestats { w } {
    global fmri

    if { !$fmri(stats_yn) } {
	$w.f.opts.nb pageconfigure stats -state disabled
    }  else {
	$w.f.opts.nb pageconfigure stats -state normal
    }
}

#}}}
#{{{ feat3:updatepoststats

proc feat3:updatepoststats { w dummy } {
    global fmri

    if { !$fmri(poststats_yn) } {
	$w.f.opts.nb pageconfigure poststats -state disabled
    } else {
	if { !$fmri(stats_yn) } {
	    set fmri(ndelete) 0
	}
	$w.f.opts.nb pageconfigure poststats -state normal
    }

    if { !$fmri(thresh) } {
	pack forget $w.f.uncorrected ; pack forget $w.f.voxel ; pack forget $w.f.z_thresh ; pack forget $w.f.prob_thresh
	pack forget $w.f.render
    } else {
	pack $w.f.render -in $fmri(lfpoststats) -after $w.f.thresh -side top -anchor w -pady 5

	if { $fmri(thresh) == 1 } {
	    pack forget $w.f.voxel ; pack forget $w.f.z_thresh ; pack forget $w.f.prob_thresh
	    pack $w.f.uncorrected -in $fmri(lfthresh) -side left
	} elseif { $fmri(thresh) == 2 } {
	    pack forget $w.f.uncorrected ; pack forget $w.f.z_thresh ; pack forget $w.f.prob_thresh
	    pack $w.f.voxel -in $fmri(lfthresh) -side left
	} else {
	    pack forget $w.f.uncorrected ; pack forget $w.f.voxel
	    pack $w.f.z_thresh $w.f.prob_thresh -in $fmri(lfthresh) -side left -anchor w -pady 2
#	    set fmri(z_thresh) 2.3 ;
	}
    }

    if { !$fmri(zdisplay) } {
	pack forget $w.f.zmin
	pack forget $w.f.zmax
    } else {
	pack $w.f.zmin $w.f.zmax -in $fmri(lfrendering) -after $w.f.zmaxmenu -side left -anchor w -padx 3 -pady 5
    }

    feat3:updateselect $w 0 0
}

#}}}

#}}}
#{{{ feat write and load

#{{{ feat3:write

proc feat3:write { w filename } {
    global fmri
    global FSLDIR
    global feat_files

    set filename [ file rootname $filename ].fsf

    set channel [ open $filename "w" ]
    puts $channel "$fmri(version) $fmri(type) $fmri(analysis) $fmri(paradigm_type)"
    puts $channel "$fmri(tr) $fmri(npts) $fmri(ndelete)"
    puts $channel "$fmri(multiple) $fmri(delay) $fmri(saveprep_yn) $fmri(savefolder_yn) $fmri(help_yn)"
    puts $channel "$fmri(filtering_yn) $fmri(brain_thresh) $fmri(mc) $fmri(sh_yn) $fmri(smooth) $fmri(norm_yn) $fmri(temphp_yn) $fmri(templp_yn) $fmri(paradigm_hp)"
    puts $channel "$fmri(stats_yn)"
    puts $channel "$fmri(evs) $fmri(ncon) $fmri(dmred) $fmri(dmgrey) $fmri(constcol)"
    puts $channel "$fmri(mf_type) $fmri(mf_cycle) $fmri(mf_file)"
    puts $channel "$fmri(poststats_yn) $fmri(thresh) $fmri(prob_thresh) $fmri(z_thresh)"
    puts $channel "$fmri(zdisplay) $fmri(zmin) $fmri(zmax) $fmri(rendertype)"

    if { $fmri(multiple) > 0 } {
	if { [ feat3:multiple_check $w 0 ] } {
	    return 1
	}
	set i 1
	while { $i <= $fmri(multiple) } {
	    puts $channel "$feat_files($i)"
	    incr i 1
	}
    }

    if { $fmri(paradigm_type)==3 } {
	set i 1
	while { $i <= $fmri(evs) } {
	    puts $channel "$fmri(shape$i) $fmri(convolve$i) $fmri(tempfilt_yn$i) $fmri(deriv_yn$i)"
	    
	    switch $fmri(shape$i) {
		0 { puts $channel " $fmri(skip$i) $fmri(off$i) $fmri(on$i) $fmri(phase$i) $fmri(stop$i)" }
		1 { puts $channel " $fmri(skip$i) $fmri(period$i) $fmri(phase$i) $fmri(stop$i)" }
		2 { puts $channel " $fmri(custom$i)" }
		3 { puts $channel " $fmri(custom$i)" }
		4 {
		    for { set j 1 } { $j < $i } { incr j 1 } {
			puts $channel " $fmri(interactions${i}.$j)"
		    }
		}
	    }
	    
	    switch $fmri(convolve$i) {
		0 { }
		1 { puts $channel " $fmri(gausssigma$i) $fmri(gaussdelay$i)" }
		2 { puts $channel " $fmri(gammasigma$i) $fmri(gammadelay$i)" }
		3 { }
	    }
	    
	incr i 1
	}
	
	set i 1
	while { $i <= $fmri(ncon) } {
	    set j 1
	    while { $j <= $fmri(evs) } {
		puts $channel "$fmri(con${i}_${j}) "
		incr j 1
	    }
	    incr i 1
	}
    }

    close $channel
    
    set result 0
    if { $fmri(paradigm_type)==3 } {

	if { $fmri(npts) <= $fmri(ndelete) || $fmri(npts) < 2 } {
	    MxPause "You need to set \"Total volumes\" to a sensible value!"
	    return 1
	}

	set result [ catch { eval exec ${FSLDIR}/bin/create_model [ file rootname $filename ] } ErrMsg ]
	if {$result != 0} {
	    puts "Problem with processing the model: $ErrMsg"
	    MxPause "Problem with processing the model: $ErrMsg"
	} else {
	    if { [ string length $ErrMsg ] > 0 } {
		puts "Problem with processing the model: $ErrMsg"
		MxPause "Problem with processing the model: $ErrMsg"
	    }
	}
	set grot [ catch { eval exec ppmtogif [ file rootname $filename ].ppm > [ file rootname $filename ].gif } ErrMsg ]
    }

    return $result
}

#}}}
#{{{ feat3:load

proc feat3:load { w full filename } {
    global fmri
    global feat_files

    set channel [ open $filename "r" ]

    gets $channel line
    scan $line "%f %d %d %d" version fmri(type) analysis fmri(paradigm_type)
    gets $channel line
    scan $line "%f %d %d" fmri(tr) fmri(npts) fmri(ndelete)
    gets $channel line
    scan $line "%d %f %d %d %d" multiple fmri(delay) fmri(saveprep_yn) fmri(savefolder_yn) fmri(help_yn)
    gets $channel line
    scan $line "%d %f %d %d %f %d %d %d %f" filtering_yn fmri(brain_thresh) fmri(mc) fmri(sh_yn) fmri(smooth) fmri(norm_yn) fmri(temphp_yn) fmri(templp_yn) fmri(paradigm_hp)
    gets $channel line
    scan $line "%d" stats_yn
    gets $channel line
    scan $line "%d %d %d %d %d" fmri(evs) fmri(ncon) fmri(dmred) fmri(dmgrey) fmri(constcol)
    gets $channel line
    scan $line "%d %f %s" fmri(mf_type) fmri(mf_cycle) fmri(mf_file)
    gets $channel line
    scan $line "%d %d %f %f" poststats_yn fmri(thresh) fmri(prob_thresh) fmri(z_thresh)
    gets $channel line
    scan $line "%d %f %f %d" fmri(zdisplay) fmri(zmin) fmri(zmax) fmri(rendertype)

    set i 1
    while { $i <= $multiple } {
	gets $channel line
	if { $full } {
	    scan $line "%s" feat_files($i)
	}
	incr i 1
    }

    if { $full } {
	set fmri(analysis) $analysis
	set fmri(multiple) $multiple
    } else {
	set fmri(npts) [ expr $fmri(npts) - $fmri(ndelete) ]
	set fmri(ndelete) 0
    }

    feat3:updatetype $w 0 0
    feat3:updateparadigm $w 0 0
    feat3:updatemf $w 0
    feat3:updateanalysis $w 0
    feat3:updateselect $w 0 0
    feat3:updatehelp $w
    feat3:updatefiltering $w
    feat3:updatemc $w 0
    feat3:updatestats $w
    feat3:updatepoststats $w 0

    if { $fmri(paradigm_type)==3 } {

	    feat3:setup_model_evs $w 0

	    set i 1
	    while { $i <= $fmri(evs) } {
		gets $channel line
		scan $line "%d %d %d %d" fmri(shape$i) fmri(convolve$i) fmri(tempfilt_yn$i) fmri(deriv_yn$i)

		gets $channel line
		switch $fmri(shape$i) {
		    0 { scan $line "%f %f %f %f %f" fmri(skip$i) fmri(off$i) fmri(on$i) fmri(phase$i) fmri(stop$i) }
		    1 { scan $line "%f %f %f %f" fmri(skip$i) fmri(period$i) fmri(phase$i) fmri(stop$i) }
		    2 { scan $line "%s" fmri(custom$i) }
		    3 { scan $line "%s" fmri(custom$i) }
		    4 {
			for { set j 1 } { $j < $i } { incr j 1 } {
			    scan $line "%d" fmri(interactions${i}.$j)
			    if { $j < [ expr $i - 1 ] } {
				gets $channel line
			    }
			}
		    }
		}

		if { $fmri(convolve$i) != 0 && $fmri(convolve$i) != 3 } {
		    gets $channel line
		    switch $fmri(convolve$i) {
			1 { scan $line "%f %f" fmri(gausssigma$i) fmri(gaussdelay$i) }
			2 { scan $line "%f %f" fmri(gammasigma$i) fmri(gammadelay$i) }
		    }
		}
		
		incr i 1
	    }

	set i 1
	while { $i <= $fmri(ncon) } {
	    set j 1
	    while { $j <= $fmri(evs) } {
		gets $channel line
		scan $line "%d" fmri(con${i}_${j})
		incr j 1
	    }
	    incr i 1
	}
    }

    close $channel
}

#}}}

#}}}
#{{{ multiple analyses popup

proc feat3:multiple_select { w } {
    global PXHOME
    global FSLDIR
    global feat_files
    global fmri

    #{{{ setup window

    set count 0
    set w0 ".dialog[incr count]"
    while { [ winfo exists $w0 ] } {
        set w0 ".dialog[incr count]"
    }

    toplevel $w0

    wm iconname $w0 "Select"
    wm iconbitmap $w0 @${FSLDIR}/tcl/fmrib.xbm

    if { $fmri(analysis) != 4 } {
	wm title $w0 "Select 4D data"
    } else {
	if { $fmri(multiple) == 1 } {
	    wm title $w0 "Select FEAT directory"
	} else {
	    wm title $w0 "Select FEAT directories"
	}
    }

    frame $w0.f

#}}}
    #{{{ setup file selections

    set i 1
    while { $i <= $fmri(multiple) } {

	tixFileEntry $w0.filename$i -label " $i:   "\
		-variable feat_files($i) \
		-options {
            entry.width 60
        }

	if { $fmri(analysis) != 4 } {
	    $w0.filename$i configure -dialogtype tixFileSelectDialog
	    $w0.filename$i filedialog subwidget fsbox config -pattern "*.hdr"
	} else {
	    $w0.filename$i configure -dialogtype tixDirSelectDialog
	    $w0.filename$i filedialog subwidget fsbox config -pattern "*.feat"
	}

	$w0.filename$i filedialog subwidget fsbox config -directory ~

	pack $w0.filename$i -in $w0.f -side top -expand yes -fill both -padx 3 -pady 3
	incr i 1
    }

#}}}
    #{{{ setup buttons

frame $w0.btns
frame $w0.btns.b -relief raised -borderwidth 1

button $w0.cancel -command "feat3:multiple_check $w 1; destroy $w0" -text "OK"

pack $w0.btns.b -side bottom -fill x -padx 3 -pady 5
pack $w0.cancel -in $w0.btns.b -side left -expand yes -padx 3 -pady 3 -fill y

#}}}
 
    pack $w0.f $w0.btns -expand yes -fill both
}

proc feat3:multiple_check { w load } {
    global feat_files
    global fmri
    global PXHOME
    global FSLDIR

    set AllOK 1
    set i 1
    while { $i <= $fmri(multiple) } {
	if { ! [ info exists feat_files($i) ] || ! [ file exists $feat_files($i) ] } {
	    set AllOK 0
	} else {
	    if { $fmri(analysis) != 4 } {
		set fmri(npts) [ exec ${FSLDIR}/bin/avwnvols [ file rootname $feat_files($i) ] 2> /dev/null ]
	    } else {
		if { ! [ file exists $feat_files($i)/design.fsf ] } {
		    set AllOK 0
		} else {
		    if { $load && $i == 1 } {
			feat3:load $w 0 $feat_files(1)/design.fsf
		    }
		}
	    }
	}
	incr i 1
    }
    
    if { ! $AllOK} {
	MxPause "Warning: you haven't filled in all the experiment selections with valid filenames"
	return 1
    }

    return 0
}

#}}}
#{{{ feat3:setup_model

proc feat3:setup_model { w } {
    global FSLDIR
    global fmri

    #{{{ setup window

    if { [ info exists fmri(w_model) ] } {
	if { [ winfo exists $fmri(w_model) ] } {
	    destroy $fmri(w_model)
	}
    }

    set count 0
    set w0 ".dialog[incr count]"
    while { [ winfo exists $w0 ] } {
        set w0 ".dialog[incr count]"
    }

    set fmri(w_model) $w0

    toplevel $w0

    wm title $w0 "General Linear Model"
    wm iconname $w0 "GLM"
    wm iconbitmap $w0 @${FSLDIR}/tcl/fmrib.xbm

    frame $w0.f

#}}}
    #{{{ setup EVs

if { $fmri(analysis) != 4 } {

    tixNoteBook $w0.f.nb

    for { set i 1 } { $i <= $fmri(evs) } { incr i 1 } {

	#{{{ setup and title for this page

    ${w0}.f.nb add ev$i -label "EV $i"
    set fmri(modelf$i) [ $w0.f.nb subwidget ev$i ]

#}}}
	
	#{{{ basic shape

    set grot $fmri(shape$i)

    tixOptionMenu $w0.f.nb.shape$i -label " Basic shape: " -variable fmri(shape$i)
    $w0.f.nb.shape$i add command 0 -label "Square"
    $w0.f.nb.shape$i add command 1 -label "Sinusoid"
    $w0.f.nb.shape$i add command 2 -label "Custom (1 entry per volume)"
    $w0.f.nb.shape$i add command 3 -label "Custom (3 column format)"
    if { $i > 2 } {
	$w0.f.nb.shape$i add command 4 -label "Interaction"
    }

    set fmri(shape$i) $grot

$w.bhelp bind $w0.f.nb.shape$i -msg "Choose the basic shape of the waveform that describes the stimulus or
confound that you wish to model. The basic waveform should be exactly
in time with the applied stimulation, i.e., not lagged at all; the
measured (time-series) response will be delyed with respect to the
stimulation, and this delay is modelled in the design matrix by
convolution of the basic waveform with a suitable haemodynamic
response function (see appropriate bubble-help).

For an on/off (or a regularly-spaced single-event) experiment this
would normally be a square wave, controlled by parameters which also
need setting up. To model single-event experiments with this method,
the \"On\" periods will probably be small - e.g., 1s or even less.

For a single-event experiment with irregular timing for the
stimulations, a custom file can be used. With \"Custom (1 entry per
volume)\", the custom file should be ASCII, and should be a list of
numbers, separated by spaces or newlines, with one number for each
volume (after subtracting the number of deleted images). These numbers
can either all be 0s and 1s, or can take a range of values. The former
case would be appropriate if the same stimulus was applied at varying
time points; the latter would be appropriate, for example, if recorded
subject responses are to be inserted as an effect to be modelled. Note
that it may or may not be appropriate to convolve this particular
waveform with an HRF - in the case of single-event, it is.

For even finer control over the input waveform, choose \"Custom (3
column format)\". In this case the custom file consists of triplets of
numbers; you can input any number of triplets. Each triplet describes
a short period of time and the value of the input function during that
time. The first number in each triplet is the onset (in seconds) of
the period, the second number is the duration (in seconds) of the
period, and the third number is the value of the input during that
period. The same comments as above apply, about whether these numbers
are 0s and 1s, or vary continuously.

Note that whilst all columns are demeaned before model fitting,
neither custom input will be rescaled. One reason is that if you have
more than one custom waveform, you may want to keep relative scalings
in the set values.

If you select \"Interaction\" then you can set which EVs this EV
models the interaction between. This EV is produced by multiplying
together selected EVs, and allows the modelling of the non-additive
interaction between the selected EVs."

#}}}
	#{{{ timings / custom name

frame $w0.f.nb.timings$i

tixControl $w0.f.nb.skip$i -label "    Skip " -variable fmri(skip$i) -step 1 -min 0 -selectmode immediate
$w.bhelp bind $w0.f.nb.skip$i -msg "The initial period (seconds) before the waveform commences."

tixControl $w0.f.nb.off$i -label "    Off " -variable fmri(off$i) -step 1 -min 0 -selectmode immediate
$w.bhelp bind $w0.f.nb.off$i -msg "The duration (seconds) of the \"Off\" periods in the square wave."

tixControl $w0.f.nb.on$i -label "    On " -variable fmri(on$i) -step 1 -min 0 -selectmode immediate
$w.bhelp bind $w0.f.nb.on$i -msg "The duration (seconds) of the \"On\" periods in the square wave."

tixControl $w0.f.nb.phase$i -label "    Phase " -variable fmri(phase$i) -step 1 -min 0 -selectmode immediate
$w.bhelp bind $w0.f.nb.phase$i -msg "The phase shift (seconds) of the waveform. By default, after the \"Skip\" period, the square wave\nstarts with a full \"Off\" period and the \"Sinusoid\" starts by falling from zero. However, the\nwave can be brought forward in time according to the phase shift."

tixControl $w0.f.nb.stop$i -label "    Stop after " -variable fmri(stop$i) -step 1 -min -1 -selectmode immediate
$w.bhelp bind $w0.f.nb.stop$i -msg "The active duration (seconds) of the waveform, starting\nafter the \"Skip\" period. \"-1\" means do not stop."

tixControl $w0.f.nb.period$i -label "    Period " -variable fmri(period$i) -step 1 -min 0 -selectmode immediate
$w.bhelp bind $w0.f.nb.period$i -msg "The period (seconds) of the \"Sinusoid\" waveform."

tixFileEntry $w0.f.nb.custom$i -label "    Filename " \
	-variable fmri(custom$i) \
	-options {
    entry.width 30
}
$w0.f.nb.custom$i filedialog subwidget fsbox config -directory "~" -pattern "*"

if { $i > 2 } {
    frame $w0.f.nb.interaction$i
    label $w0.f.nb.interaction$i.label -text "Between EVs "
    pack $w0.f.nb.interaction$i.label -in $w0.f.nb.interaction$i -side left -padx 3
    for { set j 1 } { $j < $i } { incr j 1 } {
	label $w0.f.nb.interaction$i.label$j -text "$j"
	checkbutton $w0.f.nb.interaction$i.$j -variable fmri(interactions${i}.$j) -command "feat3:setup_model_update_evs $w $w0 $i 0"
	pack $w0.f.nb.interaction$i.label$j $w0.f.nb.interaction$i.$j -in $w0.f.nb.interaction$i -side left
    }
}

#}}}

	collapsible frame $w0.f.nb.opt$i -title "Advanced"
	#{{{ convolution

set grot $fmri(convolve$i)

tixOptionMenu $w0.f.nb.convolve$i -label " Convolution: " -variable fmri(convolve$i)
$w0.f.nb.convolve$i add command 0 -label "None"
$w0.f.nb.convolve$i add command 1 -label "Gaussian"
$w0.f.nb.convolve$i add command 2 -label "Gamma"
$w0.f.nb.convolve$i add command 3 -label "SPM99 HRF"

set fmri(convolve$i) $grot

$w.bhelp bind $w0.f.nb.convolve$i -msg "The form of the HRF (haemodynamic response function) convolution that will be applied to the basic\nwaveform. This blurs and delays the original waveform, in an attempt to match the difference\nbetween the input function (original waveform, i.e., stimulus waveform) and the output function\n(measured FMRI haemodynamic response). If the original waveform is already in an appropriate form,\ne.g., was sampled from the data itself, \"None\" should be selected. The other options are all\nsomewhat similar blurring and delaying functions. \"Gaussian\" is simply a Gaussian kernel, whose\nwidth and lag can be altered. \"Gamma\" is a Gamma variate (in fact a normalisation of the\nprobability density function of the Gamma function); again, width and lag can be altered.\n\"SPM99 HRF\" is a preset function which is a mixture of two Gamma functions - a standard positive\nfunction at normal lag, and a small, delayed, inverted Gamma, which attempts to model the late\nundershoot. Unless you understand the differences between the above options, and what effect the\nnumerical parameters have, it is safest to use the default settings."

#}}}
	#{{{ convolution parameters

frame $w0.f.nb.convparams$i

tixControl $w0.f.nb.gausssigma$i -label "    Sigma "    -variable fmri(gausssigma$i) -step 0.1 -min 0 -selectmode immediate

tixControl $w0.f.nb.gaussdelay$i -label "    Peak lag " -variable fmri(gaussdelay$i) -step 0.1 -min 0 -selectmode immediate

tixControl $w0.f.nb.gammasigma$i -label "    Stddev "   -variable fmri(gammasigma$i) -step 0.1 -min 0.01 -selectmode immediate

tixControl $w0.f.nb.gammadelay$i -label "    Mean lag " -variable fmri(gammadelay$i) -step 0.1 -min 0.01 -selectmode immediate

#}}}
	#{{{ temporal filtering

frame $w0.f.nb.tempfilt$i

label $w0.f.nb.tempfilt$i.label -text "Apply temporal filtering"

checkbutton $w0.f.nb.tempfilt$i.yn -variable fmri(tempfilt_yn$i)

pack $w0.f.nb.tempfilt$i.label $w0.f.nb.tempfilt$i.yn -in $w0.f.nb.tempfilt$i -side left -padx 3

$w.bhelp bind $w0.f.nb.tempfilt$i -msg "You should normally apply the same temporal filtering to the model as
you have applied to the data, as the model is designed to look like
the data before temporal filtering was applied. Thus any extra
smoothing is correctly modelled, and long-time-scale components in the
model will be dealt with correctly."

#}}}
	#{{{ temporal derivative

frame $w0.f.nb.deriv$i

label $w0.f.nb.deriv$i.label -text "Add temporal derivative"

checkbutton $w0.f.nb.deriv$i.yn -variable fmri(deriv_yn$i)

pack $w0.f.nb.deriv$i.label $w0.f.nb.deriv$i.yn -in $w0.f.nb.deriv$i -side left -padx 3

$w.bhelp bind $w0.f.nb.deriv$i -msg "Adding a fraction of the temporal derivative of the blurred original waveform is equivalent to\nshifting the waveform slightly in time, in order to achieve a slightly better fit to the data.\nThus adding in the temporal derivative of a waveform into the design matrix allows a better fit\nfor the whole model, reducing unexplained noise, and increasing resulting statistical significances.\nNote that it is not possible to use the temporal derivative in the parameter estimates - thus in the\ngraphical design matrix display, temporal derivatives always have zero weights in the contrast vectors."

#}}}

	pack $w0.f.nb.shape$i $w0.f.nb.timings$i $w0.f.nb.opt$i -in $fmri(modelf$i) -padx 5 -pady 2 -side top -anchor w
	pack $w0.f.nb.convolve$i $w0.f.nb.convparams$i $w0.f.nb.tempfilt$i $w0.f.nb.deriv$i -in $w0.f.nb.opt$i.b -padx 5 -pady 2 -side top -anchor w

	$w0.f.nb.shape$i configure  -command " feat3:setup_model_update_evs $w $w0 $i "
	$w0.f.nb.convolve$i configure -command " feat3:setup_model_update_evs $w $w0 $i"
	$w0.f.nb.tempfilt$i.yn configure -command "feat3:setup_model_update_evs $w $w0 $i 0"
	feat3:setup_model_update_evs $w $w0 $i 0

    }

    pack $w0.f.nb -in $w0.f -padx 2
}

#}}}
    #{{{ setup contrasts

frame $w0.con
frame $w0.con.b -relief raised -borderwidth 2

tixControl $w0.con.ncon -label " Number of contrasts " \
        -variable fmri(ncon) -step 1 -min 1 -command "feat3:setup_model_update_contrasts $w $w0"

frame $w0.con.evlabelframe
label $w0.con.evlabel0 -text "     "
pack $w0.con.evlabel0 -in $w0.con.evlabelframe -side left -padx 2 -pady 2
set j 1
while { $j <= $fmri(evs) } {
    label $w0.con.evlabel${j} -text "EV$j"
    pack $w0.con.evlabel${j} -in $w0.con.evlabelframe -side left -padx 9 -pady 2
    incr j 1
}

pack $w0.con.b -in $w0.con -side bottom -fill x -padx 2 -pady 2
pack $w0.con.ncon $w0.con.evlabelframe -in $w0.con.b -side top -anchor w -padx 5 -pady 2

feat3:setup_model_update_contrasts $w $w0 0

$w.bhelp bind $w0.con.ncon -msg "

Each EV (explanatory variable, i.e., waveform) in the design matrix
results in a PE (parameter estimate) image.  This estimate tells you
how strongly that waveform fit the data at each voxel - ths higher it
is, the better the fit. For an unblurred square wave input (which will
be scaled in the model from -1 to 1), the PE image is equivalent to
0.5* the \"mean difference image\". To convert from a PE to a t
statistic image, the PE is divided by it's standard error, which is
derived from the residual noise after the complete model has been
fit. The t image is then transformed into a Z statistic via standard
statistical transformation. As well as Z images arising from single
EVs, it is possible to combine different EVs (waveforms) - for
example, to see where one has a bigger effect than another. To do
this, one PE is subtracted from another, a combined standard error is
calculated, and a new Z image is created.

All of the above is controlled by you, by setting up contrasts. Each
output Z statistic image is generated by setting up a contrast vector;
thus set the number of outputs that you want, using \"Number of
contrasts\". To convert a single EV into a Z statistic image, set it's
contrast value to 1 and all others to 0. Thus the simplest design,
with one EV only, has just one contrast vector, and only one entry in
this contrast vector; 1. To add more contrast vectors, increase the
\"Number of contrasts\". To compare two EVs, for example, to subtract
one stimulus type (EV1) from another type (EV2), set EV1's contrast
value to -1 and EV2's to 1. A Z statistic image will be generated
according to this request."

#}}}
    #{{{ setup options

frame $w0.opt
frame $w0.opt.m -relief raised -borderwidth 2

collapsible frame $w0.opt.m.coll -title "Options"

#{{{ red and grey options

frame $w0.dmred
label $w0.dmred_label -text "Use red line graphs in design picture"
checkbutton $w0.dmred_yn -variable fmri(dmred) -command "feat3:setup_model_update_dmred $w"
pack $w0.dmred_label $w0.dmred_yn -in $w0.dmred -side left

frame $w0.dmgrey
label $w0.dmgrey_label -text "Use greyscale plots in design picture"
checkbutton $w0.dmgrey_yn -variable fmri(dmgrey) -command "feat3:setup_model_update_dmgrey $w"
pack $w0.dmgrey_label $w0.dmgrey_yn -in $w0.dmgrey -side left

#}}}
#{{{ COMMENT constant column

#frame $w0.constcol
#label $w0.constcol_label -text "Include constant column in design matrix"
#checkbutton $w0.constcol_yn -variable fmri(constcol)
#pack $w0.constcol_label $w0.constcol_yn -in $w0.constcol -side left
#
#$w.bhelp bind $w0.constcol_label -msg "
#
#The constant column in the design matrix accounts (in the model
#fitting) both for the mean value of the data and any non-zero-mean
#columns in the rest of the design matrix. Note that the data is
#demeaned during the model fitting, so the first reason should not be
#necessary. You should not turn this off unless you know exactly what
#you are doing in the rest of the design matrix. One situation when you
#might want to is if you account for all possiblities in a factor
#analysis, and therefore do not want a constant column, as it is a
#linear combination of the other columns anyway."
#

#}}}

#pack $w0.constcol $w0.dmred $w0.dmgrey -in $w0.opt.m.coll.b -side top -padx 3 -anchor w
pack $w0.dmred $w0.dmgrey -in $w0.opt.m.coll.b -side top -padx 3 -anchor w

pack $w0.opt.m.coll -in $w0.opt.m -side top -anchor w
pack $w0.opt.m -in $w0.opt -side bottom -fill x -padx 2 -pady 2

#}}}
    #{{{ setup buttons

frame $w0.btns
frame $w0.btns.b -relief raised -borderwidth 2

button $w0.btns.view -command "feat3:setup_model_preview $w" -text "Preview"
button $w0.btns.cancel -command "feat3:setup_model_destroy $w $w0" -text "Done"

pack $w0.btns.view $w0.btns.cancel -in $w0.btns.b -side left -expand yes -padx 3 -pady 3 -fill y
pack $w0.btns.b -in $w0.btns -side bottom -fill x -padx 2 -pady 2

#}}}
 
    pack $w0.f $w0.con $w0.opt $w0.btns -expand yes -fill both -padx 2 -pady 2
}

#{{{ feat3:setup_model_init

proc feat3:setup_model_init { w } {
    global fmri

    set fmri(ncon) 1

    set fmri(constcol) 0

    set fmri(dmred) 1
    set fmri(dmgrey) 1

    feat3:setup_model_evs $w 0
}

#}}}
#{{{ feat3:setup_model_evs

proc feat3:setup_model_evs { w dummy } {
    global fmri

    set i 1
    while { $i <= $fmri(evs) } {
	if { ! [ info exists fmri(shape$i) ] } {
	    set fmri(shape$i) 0
	    set fmri(skip$i) 0
	    set fmri(off$i) 30
	    set fmri(on$i) 30
	    set fmri(phase$i) 0
	    set fmri(stop$i) -1
	    set fmri(period$i) 60
	    set fmri(convolve$i) 2
	    set fmri(gausssigma$i) 2.8
	    set fmri(gaussdelay$i) 5
	    set fmri(gammasigma$i) 3
	    set fmri(gammadelay$i) 6
	    set fmri(tempfilt_yn$i) 1
	    set fmri(deriv_yn$i) 1

	    if { $i > 2 } {
		for { set j 1 } { $j < $i } { incr j 1 } {
		    if { $j < 3 } {
			set fmri(interactions${i}.$j) 1
		    } else {
			set fmri(interactions${i}.$j) 0
		    }
		}
	    }
	}

	incr i 1
    }

    if { [ winfo exists $fmri(w_model) ] } {
	feat3:setup_model $w
    }
}

#}}}
#{{{ feat3:setup_model_vars_simple

proc feat3:setup_model_vars_simple { w } {
    global fmri

    if { $fmri(paradigm_type) == 1 } {

	set fmri(evs) 1

	set fmri(shape1) 0
	set fmri(skip1) 0
	set fmri(off1) $fmri(a_count)
	set fmri(on1) $fmri(b_count)
	set fmri(phase1) 0
	set fmri(stop1) -1
	set fmri(convolve1) 2
	set fmri(gammasigma1) 3
	set fmri(gammadelay1) 6
	set fmri(tempfilt_yn1) 1
	set fmri(deriv_yn1) 1
	
	set fmri(ncon) 1
	set fmri(con1_1) 1

	set fmri(paradigm_hp) [ expr 1.5 * ( $fmri(a_count) + $fmri(b_count) ) ]

    } elseif { $fmri(paradigm_type) == 2 } {

	set fmri(evs) 2

	set fmri(shape1) 0
	set fmri(shape2) 0
	set fmri(skip1) 0
	set fmri(skip2) 0
	set fmri(off1) [ expr $fmri(a_count) * 2 + $fmri(c_count) ]
	set fmri(on1) $fmri(b_count)
	set fmri(off2) [ expr $fmri(a_count) * 2 + $fmri(b_count) ]
	set fmri(on2) $fmri(c_count)
	set fmri(phase1) [ expr $fmri(a_count) + $fmri(c_count) ]
	set fmri(phase2) 0
	set fmri(stop1) -1
	set fmri(stop2) -1
	set fmri(convolve1) 2
	set fmri(convolve2) 2
	set fmri(gammasigma1) 3
	set fmri(gammasigma2) 3
	set fmri(gammadelay1) 6
	set fmri(gammadelay2) 6
	set fmri(tempfilt_yn1) 1
	set fmri(tempfilt_yn2) 1
	set fmri(deriv_yn1) 1
	set fmri(deriv_yn2) 1
	
	set fmri(ncon) 5
	set fmri(con1_1) 1
	set fmri(con1_2) 0
	set fmri(con2_1) 0
	set fmri(con2_2) 1
	set fmri(con3_1) 1
	set fmri(con3_2) -1
	set fmri(con4_1) -1
	set fmri(con4_2) 1
	set fmri(con5_1) 1
	set fmri(con5_2) 1

	set fmri(paradigm_hp) [ expr 1.5 * ( ( 2 * $fmri(a_count) ) + $fmri(b_count) + $fmri(c_count) ) ]

    }
}

#}}}
#{{{ feat3:setup_model_update_evs

proc feat3:setup_model_update_evs { w w0 i dummy } {
    global fmri

    pack forget $w0.f.nb.skip$i $w0.f.nb.off$i $w0.f.nb.on$i $w0.f.nb.phase$i $w0.f.nb.stop$i $w0.f.nb.period$i $w0.f.nb.custom$i $w0.f.nb.interaction$i

    if { $fmri(shape$i) == 0 } {
	pack $w0.f.nb.skip$i $w0.f.nb.off$i $w0.f.nb.on$i $w0.f.nb.phase$i $w0.f.nb.stop$i -in $w0.f.nb.timings$i -padx 5 -pady 2 -side top -anchor w
    } elseif { $fmri(shape$i) == 1 } {
	pack $w0.f.nb.skip$i $w0.f.nb.period$i $w0.f.nb.phase$i $w0.f.nb.stop$i -in $w0.f.nb.timings$i -padx 5 -pady 2 -side top -anchor w
    } elseif { $fmri(shape$i) == 2 || $fmri(shape$i) == 3 } {
	pack $w0.f.nb.custom$i -in $w0.f.nb.timings$i -padx 5 -pady 2 -side top -anchor w
    } elseif { $fmri(shape$i) == 4 } {
	pack $w0.f.nb.interaction$i -in $w0.f.nb.timings$i -padx 5 -pady 2 -side top -anchor w
	set selected 0
	for { set j 1 } { $j < $i } { incr j 1 } {
	    set selected [ expr $selected + $fmri(interactions${i}.$j) ]
	}
	if { $selected < 2 } {
	    if { !$fmri(interactions${i}.1) } {
		set fmri(interactions${i}.1) 1
	    } else {
		set fmri(interactions${i}.2) 1
	    }
	}
    }

    pack forget $w0.f.nb.gausssigma$i $w0.f.nb.gaussdelay$i $w0.f.nb.gammasigma$i $w0.f.nb.gammadelay$i

    if { $fmri(shape$i) == 4 } {
	set fmri(convolve$i) 0
	set fmri(tempfilt_yn$i) 0
    }

    if { $fmri(convolve$i) == 1 } {
	pack $w0.f.nb.gausssigma$i $w0.f.nb.gaussdelay$i -in $w0.f.nb.convparams$i -padx 5 -pady 2 -side top -anchor w
    } elseif { $fmri(convolve$i) == 2 } {
	pack $w0.f.nb.gammasigma$i $w0.f.nb.gammadelay$i -in $w0.f.nb.convparams$i -padx 5 -pady 2 -side top -anchor w
    }
}

#}}}
#{{{ feat3:setup_model_update_dmred,dmgrey

proc feat3:setup_model_update_dmred { w } {
    global fmri

    if { $fmri(dmred) == 0 } {
	set fmri(dmgrey) 1
    }
}

proc feat3:setup_model_update_dmgrey { w } {
    global fmri

    if { $fmri(dmgrey) == 0 } {
	set fmri(dmred) 1
    }
}

#}}}
#{{{ feat3:setup_model_update_contrasts

proc feat3:setup_model_update_contrasts { w w0 dummy } {
    global fmri

    set i 1
    while { $i > 0 } {
	if { [ winfo exists $w0.con.frame$i ] } {
	    pack forget $w0.con.frame$i
	    incr i 1
	} else {
	    set i -1
	}
    }

    set i 1
    while { $i <= $fmri(ncon) } {

	if { ! [ winfo exists $w0.con.frame$i ] } {
	    frame $w0.con.frame$i

	    label $w0.con.label$i -text " C$i "
	    pack $w0.con.label$i -in $w0.con.frame$i -side left -padx 2 -pady 2
	    $w.bhelp bind $w0.con.label$i -msg "Contrast vector number $i - this will result in Z statistic image number $i"

	    set j 1
	    while { $j <= $fmri(evs) } {

		if { ! [ info exists fmri(con${i}_${j}) ] } {
		    set fmri(con${i}_${j}) 0
		    if { $i==1 && $j==1 } {
			set fmri(con${i}_${j}) 1
		    }
		}

		tixControl $w0.con.con${i}_${j} -variable fmri(con${i}_${j}) -step 1 \
			-selectmode immediate -options { entry.width 2 }
		pack $w0.con.con${i}_${j} -in $w0.con.frame$i -side left -padx 2 -pady 2
		$w.bhelp bind $w0.con.con${i}_${j} -msg "The weight given to EV$j within contrast vector $i."
		incr j 1
	    }
	}

	pack $w0.con.frame$i -in $w0.con.b -side top -anchor w

	incr i 1
    }
}

#}}}
#{{{ feat3:setup_model_preview

proc feat3:setup_model_preview { w } {
    global fmri

    set problem [ feat3:write $w $fmri(feat_filename) ]
    if { $problem } {
	return 1
    }

    set count 0
    set w1 ".dialog[incr count]"
    while { [ winfo exists $w1 ] } {
        set w1 ".dialog[incr count]"
    }

    toplevel $w1
    wm title $w1 "Model"
    wm iconname $w1 "Model"

    set graphpic [ image create photo -file [ file rootname $fmri(feat_filename) ].ppm ]

    button $w1.btn -command "destroy $w1" -image $graphpic -borderwidth 0
    pack $w1.btn -in $w1

    $w.bhelp bind $w1.btn -msg "

This is a graphical representation of the design matrix and parameter
contrasts.

The bar on the left is a representation of time, which starts at the
top and points downwards. The white marks show the position of every
10th volume in time. The red bar shows the period of the longest
temporal cycle which was passed by the highpass filtering.

The main top part shows the design matrix; time is represented on the
vertical axis and each column is a different explanatory variable
(e.g., stimulus type). Both the red lines and the black-white images
represent the same thing - the variation of the waveform in time.

The bottom part shows the requested contrasts; each column refers to
the weighting of the relevant explanatory variable (often either just
1, 0 or -1), and each row is a different contrast vector. Thus each
row will result in it's own Z statistic image.

Click anywhere to dismiss this window."

  return 0
}

#}}}
#{{{ feat3:setup_model_destroy

proc feat3:setup_model_destroy { w w0 } {
    global fmri

    if { ! [ feat3:setup_model_preview $w ] } {
	destroy $w0
    }
}

#}}}

#}}}
#{{{ call GUI and wait

feat3 .rename
tkwait window .rename

#}}}
