#

#{{{ copyright

# FEAT - FMRIB's Easy Analysis Tool for FMRI
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

#}}}
#{{{ setups

set VERSION 3.12

# for exported versions don't try to renice FEAT sessions
if { [ file exists /usr/local/bin/nicemedx ] } {
    set RENICE 1
} else {
    set RENICE 0
}

if { ! [ info exists INMEDX ] } {
    set INMEDX 1
}

#}}}

proc feat3_proc { featname popups FeatDirName } {
    #{{{ Setup variables

set thedate [exec date]
puts "Starting FEAT at $thedate"

global PXHOME FSLDIR HOME USER PROCID HOSTNAME OS VERSION MEDXV CLUSTERRSH

MxShowInformationalPopups 0

# set no image history and don't save old versions (which shouldn't now exist) to disk
MxSetImageHistoryDefaults 0 false

# set tile pool correctly if the user has a small one
MxGetTilePoolDefaults currentsize
if { $currentsize < 100 } {
    MxSetTilePoolDefaults 150
}

MxGetCurrentFolder F0
MxGetPageByName $F0 "FirstPage" FirstPage

#}}}
    #{{{ Read FEAT setup file

    set channel [ open ${featname}.fsf "r" ]

    gets $channel line
    scan $line "%f %d %d %d" fileversion type analysis paradigm_type
    gets $channel line
    scan $line "%f %d %d" tr npts ndelete
    gets $channel line
    scan $line "%d %f %d %d %d" multiple delay saveprep_yn savefolder_yn help_yn
    gets $channel line
    scan $line "%d %f %d %d %f %d %d %d %f" filtering_yn brain_thresh mc sh_yn smooth norm_yn temphp_yn templp_yn paradigm_hp
    gets $channel line
    scan $line "%d" stats_yn
    gets $channel line
    scan $line "%d %d %d %d %d" evs ncon dmred dmgrey constcol
    gets $channel line
    scan $line "%d %f %s" mf_type mf_cycle mf_file
    gets $channel line
    scan $line "%d %d %f %f" poststats_yn thresh prob_thresh z_thresh
    gets $channel line
    scan $line "%d %f %f %d" zdisplay zmin zmax rendertype

    close $channel

#}}}
    #{{{ Delete images

if { $analysis != 4 } {

    MxGetCurrentGroup Group
    MxGetPagesFromGroup $Group GPages
    set total_volumes [ llength $GPages ]
    puts "Total original volumes = $total_volumes"
    if { $total_volumes != $npts } {
	puts "Error - current group has different number of time points to what was entered"
	return 1
    }

    if { $ndelete > 0 } {
	puts "Deleting pages from loaded group - BE WARNED for future analysis!"
    }

    set i 0
    while { $i < $ndelete } {
	set Todelete [ lindex $GPages $i ]
	puts "Deleting page $Todelete"
	MxDeletePage { $Todelete }
	incr i 1
    }

    MxGetPagesFromGroup $Group GPages
    set total_volumes [ llength $GPages ]
    MxSetTimeSeriesProperties $Group 1 red

}

#}}}
    #{{{ Get group file name and setup folder details

if { $analysis != 4 } {
    MxGetPageProperties $Group GroupProps

    set GroupFullNameTmp [ keylget GroupProps Name ]
    regsub -all " " $GroupFullNameTmp "" GroupFullNameTmp2
    regsub -all "\"" $GroupFullNameTmp2 "" GroupFullName

    set GroupName     [ file rootname $GroupFullName ]
    set GroupDirName  [ file dirname  $GroupFullName ]
    set GroupFileName [ file tail     $GroupFullName ]

    set IsTmp [ string compare /tmp [ string range $GroupDirName 0 3 ] ]
    
    if { ! [ file writable "${GroupDirName}" ] || $IsTmp == 0 } {
	set GroupName [ file rootname [ file rootname [ file rootname "${HOME}/${GroupFileName}" ] ] ]
    }

    set FeatDirName [ new_filename ${GroupName}.feat ]
    exec /bin/mkdir $FeatDirName
}

set CLUSTER  ${FeatDirName}/cluster
set STATSDIR ${FeatDirName}/stats

set FOLDER   [ file rootname [ new_filename ${FeatDirName}/folder.fld ] ]
MxSetFolderProperties $F0 "{Name ${FOLDER}} {File ${FOLDER}.fld} {SaveAll true}"

set DESIGN [ file rootname [ new_filename ${FeatDirName}/design.fsf ] ]

set PLUSES [ string trimleft [ file tail $DESIGN ] design ]

exec /bin/cp ${featname}.fsf ${DESIGN}.fsf
if { $paradigm_type == 3 } {
    exec /bin/cp ${featname}.mat ${DESIGN}.mat
    exec /bin/cp ${featname}.con ${DESIGN}.con
    exec /bin/cp ${featname}.gif ${DESIGN}.gif
}

#}}}
    #{{{ Select target_vol

if { $analysis != 4 } {

    # choose image 10
    set target_vol_number 9

    # but not if there aren't 10 images in group
    if { $target_vol_number >= $total_volumes } {
	set target_vol_number [ expr $total_volumes - 1 ]
    }

    # but choose 0 if doing SPM MC (cos that's what it uses)
    if { $mc == 4 && $filtering_yn == 1 } {
	set target_vol_number 0
    }

    # now create target_vol, get its props, name it and move it to top of folder
    MxDupPage [ lindex $GPages $target_vol_number ] target_vol
    MxGetImageProperties $target_vol target_props
    name_page $target_vol "Example functional image"
    MxCutPage $target_vol
    MxPastePage $FirstPage target_vol

    MxSaveImageAs $target_vol AVW ${FeatDirName}/example_func.hdr true

} else {

    MxOpenImage $F0 ${FeatDirName}/example_func.hdr target_vol
    MxGetImageProperties $target_vol target_props

}

#}}}
    #{{{ Write starts of log and report files

set logout [ open ${FeatDirName}/report${PLUSES}.log "w" ]
set report [ open ${FeatDirName}/report${PLUSES}.html "w" ]

fconfigure $logout -buffering none
fconfigure $report -buffering none

puts $logout "Started FEAT at $thedate on $HOSTNAME"

puts $report "<HTML>

<TITLE>FEAT Report</TITLE>

<BODY BACKGROUND=\"file:${FSLDIR}/doc/images/fsl-bg\">

<hr><CENTER>
<H1>FEAT Report</H1>
${FeatDirName}/report${PLUSES}.html
</CENTER>
"

set ps "<hr><H2>Analysis methods</H2>
<p>Analysis was carried out using FEAT, the FMRIB Easy Analysis Tool, an extension of MEDx \[MEDx\]."

set rs "<p><b>References</b><br>
\[MEDx\] Sensor Systems, VA, USA.<br>
"

#}}}
    #{{{ Pre-stats

#{{{ find threshold

# should really make this more robust

MxQuietStatistics $target_vol { Maximum Minimum } false Stats
set McMin [ keylget Stats Minimum ]
set McMax [ keylget Stats Maximum ]

set absbrainthresh [ expr ( ( $McMax - $McMin ) * $brain_thresh / 100.0 ) + $McMin ]

puts "Brain intensity threshold $absbrainthresh"
puts $logout "Brain intensity threshold $absbrainthresh"

#}}}

MxSetCurrentPage $target_vol

if { $analysis != 4 } {
    if { $filtering_yn } {

	set ps "$ps The following pre-statistics processing was applied"

	#{{{ Motion correction

#mc: 0=none 1=guess 2=2DAIR 3=3DAIR 4=SPM

if { $mc > 0 } {

    if { $mc == 1 } {
	#{{{ guess MC type from number of slices

if { [ keylget target_props Slices ] < 7 } {
    set mc 2
} else {
    set mc 3
}

#}}}
    }

    if { $mc == 2 } {
	#{{{ 2D AIR

	MxAlignLinear $target_vol $Group 23 "{Interpolation 1} {StdThr $absbrainthresh} {ResliceThr $absbrainthresh} {StdPartitions 1} {ReslicePartitions 1} {Convergence 1.0000000000000001e-05} {RepeatedIter 25} {HaltAfterIter 5} {AlterAfterIter 6} {InitSampling 81} {FinalSampling 1} {SamplingDecr 3} {StdXSmooth 0} {StdYSmooth 0} {StdZSmooth 0} {ResliceXSmooth 0} {ResliceYSmooth 0} {ResliceZSmooth 0} {Directory /tmp} {Cost stddev} {pre-interp false} {NonInteract true}" NewGroup
	set ps "$ps; 2D motion correction, using AIR \[AIR\]"
	set rs "$rs\[AIR\] R.P. Woods, S.T. Grafton, C.J. Holmes, S.R. Cherry and J.C. Mazziotta. Automated image registration: I. General methods and intrasubject, intramodality validation. Journal of Computer Assisted Tomography 1998:22(141-154).<br>
"

#}}}
    } elseif { $mc == 3 } {
	#{{{ 3D AIR

	MxAlignLinear $target_vol $Group 6 "{Interpolation 1} {StdThr $absbrainthresh} {ResliceThr $absbrainthresh} {StdPartitions 1} {ReslicePartitions 1} {Convergence 1.0000000000000001e-05} {RepeatedIter 25} {HaltAfterIter 5} {AlterAfterIter 6} {InitSampling 81} {FinalSampling 1} {SamplingDecr 3} {StdXSmooth 0} {StdYSmooth 0} {StdZSmooth 0} {ResliceXSmooth 0} {ResliceYSmooth 0} {ResliceZSmooth 0} {Directory /tmp} {Cost stddev} {pre-interp false} {NonInteract true}" NewGroup
	set ps "$ps; 3D motion correction, using AIR \[AIR\]"
	set rs "$rs\[AIR\] R.P. Woods, S.T. Grafton, C.J. Holmes, S.R. Cherry and J.C. Mazziotta. Automated image registration: I. General methods and intrasubject, intramodality validation. Journal of Computer Assisted Tomography 1998:22(141-154).<br>
"

#}}}
    } elseif { $mc == 4 } {
	#{{{ SPM

	set sh 0
	set shstring ""
	if { $sh_yn == 1 } {
	    if { $tr > 4 } {
		if { $total_volumes > 40 } {
		    set sh 2
		} elseif { $total_volumes > 30 } {
		    set sh 1
		}
	    } else {
		if { $total_volumes > 40 } {
		    set sh 3
		}
		if { $tr < 2 && $total_volumes > 64 } {
		    set sh 4
		}
	    }
	    set shstring "with adjustment for movement "
	}
	puts "SPM MC adjustment $sh"
	puts $logout "SPM MC adjustment $sh"
	
	keylset SPMProps CoregInterp 1
	keylset SPMProps ResliceInterp 1
	keylset SPMProps Reg_Reslice Both
	keylset SPMProps Output 1
	keylset SPMProps NumIterations 8
	keylset SPMProps Mask true
	keylset SPMProps UsePrevXForms false
	keylset SPMProps fMRIAdjust $sh
	keylset SPMProps ReportDir "/tmp"

	MxSPMRealign $Group SPMProps NewGroup

	set ps "$ps; motion correction using the SPM realign procedure $shstring\[SPM1\]"
	set rs "$rs\[SPM1\] K.J. Friston, J. Ashburner, J.-B. Poline, C.D. Frith, J.D. Heather and R.S.J. Frackowiak. Spatial Registration and Normalization of Images. Human Brain Mapping 1995:2(165-189).<br>
"

#}}}
    }

    set Group $NewGroup
    MxGetPagesFromGroup $Group GPages
    set total_volumes [ llength $GPages ]
    MxSetTimeSeriesProperties $Group 1 red
}

#}}}

	set ip_command ""
	#{{{ Spatial filtering

if { $smooth > 0.01 } {
    set ip_command "$ip_command -s [ expr $smooth / 2.355 ]"
    set ps "$ps; spatial smoothing using a Gaussian kernel of FWHM ${smooth}mm"
}

#}}}
	#{{{ Global intensity normalization

set global_mean 10000

if { $norm_yn} {
    set ip_command "$ip_command -i $global_mean"
    set ps "$ps; global (volumetric) multiplicative mean intensity renormalization"
} else {
    set ip_command "$ip_command -I $global_mean"
    set ps "$ps; mean-based intensity normalisation of all volumes by the same factor"
}

#}}}
	#{{{ Temporal filtering

if { $temphp_yn || $templp_yn } {

    set hp_sigma_vol -1
    if { $temphp_yn } {
	if { $paradigm_type == 4 } {
	    set paradigm_hp [ expr $mf_cycle * $tr * 1.5 ]
	}
	set hp_sigma_sec [ expr $paradigm_hp / 2.0 ]
	set hp_sigma_vol [ expr $hp_sigma_sec / $tr ]
	set ps "$ps; nonlinear highpass temporal filtering (Gaussian-weighted LSF straight line fitting, with sigma=${hp_sigma_sec}s)"
    }

    set lp_sigma_vol -1
    if { $templp_yn } {
	set lp_sigma_sec 2.8
	set lp_sigma_vol [ expr $lp_sigma_sec / $tr ]
	set ps "$ps; Gaussian lowpass temporal filtering HWHM ${lp_sigma_sec}s"
    }

    set ip_command "$ip_command -t $hp_sigma_vol $lp_sigma_vol"

}

#}}}
	#{{{ Call ip and reload data

set ps "$ps."

MxSave3DGroupAs $Group AVW ${FeatDirName}/prefiltered_func_data.hdr true

puts "Filtering: $ip_command"
puts $logout "Filtering: $ip_command"

set absbrainthresh [ exec ${FSLDIR}/bin/ip ${FeatDirName}/prefiltered_func_data ${FeatDirName}/filtered_func_data $brain_thresh $ip_command | grep THRESHOLD | awk {{ print $2 }} - ]

exec /bin/rm -f ${FeatDirName}/prefiltered_func_data.hdr ${FeatDirName}/prefiltered_func_data.img

MxSetCurrentPage $target_vol

MxOpenImage $F0 ${FeatDirName}/filtered_func_data.hdr NewGroup

set Group $NewGroup
MxGetPagesFromGroup $Group GPages
set total_volumes [ llength $GPages ]
MxSetTimeSeriesProperties $Group 1 red

# following hack so that if the file is later deleted, MEDx still works
MxComputeDisplayRange [ lindex $GPages 0 ] V Min Max

#}}}

    } else {
	MxSave3DGroupAs $Group AVW ${FeatDirName}/filtered_func_data.hdr true
    }
}

MxSetCurrentPage $target_vol

#}}}
    #{{{ Stats

if { $stats_yn || $analysis == 4 } {

    if { $analysis != 4 } {
	if { $paradigm_type == 3 } {
	    #{{{ Get FILM going

if { $type == 2 } {
    set film_opts "-sa -ms 5"
} else {
    set film_opts "-gl"
}

set thecommand "$CLUSTERRSH $FSLDIR/bin/film_ols -rn $STATSDIR $film_opts -sp $FSLDIR/bin/susan_smooth ${FeatDirName}/filtered_func_data ${DESIGN}.mat $absbrainthresh"
puts "Running $thecommand in the background"
set cpid [ open "| $thecommand" r ]
fconfigure $cpid -blocking false

#}}}
	}
	#{{{ Generate binary mask for stats

MxGenerateMask $Group $brain_thresh Mask
MxSaveImageAs $Mask AVW ${FeatDirName}/mask.hdr true

#}}}
	#{{{ Make fake Z score

set AList ""
lappend AList [ lindex $GPages 0 ]
lappend AList [ lindex $GPages 1 ]
MxGroupVolumes $AList "A" A

set BList ""
set i 2
while { $i < $total_volumes } {
    lappend BList [ lindex $GPages $i ]
    incr i 1
}
MxGroupVolumes $BList "B" B

MxBetweenGrpStatistics $A $B U blah blah2 blah3 FinalZscore ResultGroup

MxGetPagesFromGroup $ResultGroup RP
set RPList " [ lindex $RP 0 ] [ lindex $RP 1 ] [ lindex $RP 2 ] [ lindex $RP 3 ] [ lindex $RP 4 ] [ lindex $RP 6 ] [ lindex $RP 7 ] "
MxDeletePage { $RPList }
MxDeleteGroupPages { $A $B $ResultGroup } No

#}}}
    } else {
	MxOpenImage $F0 ${FeatDirName}/mask.hdr Mask
	#{{{ Make fake Z score

MxDupPage $target_vol dummy

MxGroupVolumes "$target_vol $dummy" "A" A

MxSetCurrentPage $Mask
MxBetweenGrpStatistics $A $A U blah blah2 blah3 FinalZscore ResultGroup

MxGetPagesFromGroup $ResultGroup RP
set RPList " [ lindex $RP 0 ] [ lindex $RP 1 ] [ lindex $RP 2 ] [ lindex $RP 3 ] [ lindex $RP 4 ] [ lindex $RP 6 ] [ lindex $RP 7 ] "
MxDeletePage { $RPList }
MxDeleteGroupPages { $A $ResultGroup $dummy } No

#}}}
    }

    if { $paradigm_type == 3 } {
	#{{{ Wait for FILM to finish and run contrasts

if { $analysis != 4 } {
    if { $popups } {
	ScriptUpdate "Running FILM - FMRIB's Improved Linear Model"
    }

    while { ! [ eof $cpid ] } {
	gets $cpid dummy
	after 10000
    }
    
    if { $popups } {
	CancelScriptUpdate
    }
}

set PLUSSUFFIX ""
if { [ string length $PLUSES ] > 0 } {
    set PLUSSUFFIX "-suffix $PLUSES"
}
set thecommand "exec $FSLDIR/bin/contrast_mgr $PLUSSUFFIX $STATSDIR ${DESIGN}.con"
puts "$thecommand"
set result [ catch { eval $thecommand >& ${STATSDIR}.txt } ErrMsg ]

exec /bin/mv -f ${STATSDIR}.txt $STATSDIR

#}}}
	#{{{ load in results

set i $ncon
while { $i >= 1 } {
    MxOpenImage $F0 ${STATSDIR}/zstat${PLUSES}${i}.hdr Tmp
    MxDupPage $FinalZscore ZS($i)
    MxExpr $ZS($i) = " $Tmp "
    MxComputeDisplayRange $ZS($i) V min max
    name_page $ZS($i) "zstat${PLUSES}$i"

    MxOpenImage $F0 ${STATSDIR}/cope${PLUSES}${i}.hdr Tmp2
    MxDupPage $FinalZscore PE($i)
    MxExpr $PE($i) = " $Tmp2 "
    MxComputeDisplayRange $PE($i) V min max
    name_page $PE($i) "cope${PLUSES}$i"

    MxDeletePage { $Tmp $Tmp2 }
    incr i -1
}

MxDeletePage { $FinalZscore }

#}}}
	#{{{ paragraph output

set ps "$ps Statistical analysis was carried out using FILM (FMRIB's Improved Linear Model) with local autocorrelation correction \[FILM\]."
set rs "$rs\[FILM\] M. Woolrich, J.M. Brady, B. Ripley and S.M. Smith. Robust Treatment of Temporal Autocorrelations in Univariate Linear Modelling of FMRI Data. In preparation 2000.<br>
"

#}}}
    } else {
	set ncon 1
	#{{{ Call model-free

if { $analysis != 4 } {
    set ncon 1

    MxSetCurrentPage $Group

    if { $popups } {
	ScriptUpdate "Running model-free ANOVA"
    }

    exec /bin/mkdir $STATSDIR

    if { $mf_type == 1 } {
	set thecommand "exec ${FSLDIR}/bin/medx_anova ${FeatDirName}/filtered_func_data ${STATSDIR}/fstat1 ${STATSDIR}/zstat1 $brain_thresh 0 [ expr int($mf_cycle) ]"
    } else {
	set mf_events [ exec wc -l $mf_file | awk {{ print $1 }} - ]
	puts "Found $mf_events events in model-free events file."
	set thecommand "exec ${FSLDIR}/bin/medx_anovanc ${FeatDirName}/filtered_func_data ${STATSDIR}/fstat1 ${STATSDIR}/zstat1 $brain_thresh $mf_events [ expr int($mf_cycle) ] $mf_file"
    }

    puts $thecommand
    set result [ catch { eval $thecommand >& ${STATSDIR}/mf.txt } ErrMsg ]

    if { $popups } {
	CancelScriptUpdate
    }
} else {
    exec /bin/cp -f ${STATSDIR}/zstat1.hdr ${STATSDIR}/zstat${PLUSES}1.hdr
    exec /bin/cp -f ${STATSDIR}/zstat1.img ${STATSDIR}/zstat${PLUSES}1.img
}

set paradigm_type 1

#}}}
	#{{{ load in results

MxOpenImage $F0 ${STATSDIR}/zstat${PLUSES}1.hdr Tmp
MxDupPage $FinalZscore ZS(1)
MxExpr $ZS(1) = " $Tmp "
MxComputeDisplayRange $ZS(1) V min max
name_page $ZS(1) "zstat${PLUSES}1"
MxDeletePage { $Tmp }
MxDeletePage { $FinalZscore }

#}}}
	#{{{ paragraph output

set ps "$ps Statistical analysis was carried out using the model-free ANOVA method \[ANOVA\]."
set rs "$rs\[ANOVA\] S. Clare, M. Humberstone, J. Hykin, L. Blumhardt, R. Bowtell and P. Morris. Detecting Activations in Event-Related fMRI Using Analysis of Variance. Magnetic Resonance in Medicine 1999:42(1117-1122).<br>
"

#}}}
    }
}

#}}}
    #{{{ Poststats

if { $poststats_yn } {
    #{{{ setup zmin and zmax to initial extremes if doing autoscale

    if { $zdisplay == 0 } {
	set zmin 1000
	set zmax -1000
    }

#}}}
    puts "thresholding is $thresh"
    puts $logout "thresholding is $thresh"
    for { set i 1 } { $i <= $ncon } { incr i 1 } {
	#{{{ mask Z stat images

MxExpr $ZS($i) = " ( $ZS($i) * $Mask ) / 255.0 "

#}}}
	#{{{ Thresholding

#{{{ get name of stats image

MxGetImageProperties $ZS($i) stats_props
set stats_name [keylget stats_props Name]
regsub -all " " $stats_name "_" stats_name_fixed

puts "$stats_name:"
puts $logout "$stats_name:"

#}}}
#{{{ Find Z max

MxQuietStatistics $ZS($i) { Maximum } false Stats

set MaximumZ($i) [ keylget Stats Maximum ]

puts "Original MaximumZ =$MaximumZ($i)"
puts $logout "Original MaximumZ =$MaximumZ($i)"

#}}}
#{{{ save raw Z score to file if required

if { $paradigm_type == 4 } {
    MxSaveImageAs $ZS($i) AVW ${FeatDirName}/raw_${stats_name_fixed}.hdr true
}

#}}}

if { $thresh > 0 } {
    #{{{ paragraph output

if { $i == 1 } {
    if { $ncon == 1 } {
	set ps "$ps A Z (Gaussianised T) statistic image was thresholded"
    } else {
	set ps "$ps Z (Gaussianised T) statistic images were thresholded"
    }
}

#}}}
    #{{{ Duplicate Z stat image

    MxDupPage $ZS($i) ZS($i)
    name_page $ZS($i) "thresh_zstat${PLUSES}$i"

#}}}

    if { $thresh == 1 } {
	# voxel uncorrected thresholding
	#{{{ Find corrected thresholds and paragraph output

if { $i == 1 } {
    set ps "$ps at P=${prob_thresh} (uncorrected)."

    set prob_thresh [expr $prob_thresh * 2]
    if {$prob_thresh > 0.99} {
	set prob_thresh 0.99
    }
}

MxComputeUncorrectedCriticalThr $Mask U $prob_thresh dummy1 dummy2 zuncorrected dummy3
puts "zuncorrected=$zuncorrected"
puts $logout "zuncorrected=$zuncorrected"

#}}}
	#{{{ Use corrected Z threshold to mask Z image

MxMapPixels $ZS($i) -50 $zuncorrected 0 0

MxQuietStatistics $ZS($i) { Maximum } false Stats
set MaximumZ($i) [ keylget Stats Maximum ]
puts "MaximumZ =$MaximumZ($i) after thresholding"
puts $logout "MaximumZ =$MaximumZ($i) after thresholding"

#}}}
    } else {
	if { $thresh == 2 } {
	    # voxel resel corrected thresholding
	    #{{{ Paragraph output and fix prob_thresh

if { $i == 1 } {
    set ps "$ps using resel (corrected Bonferroni) thresholding with a corrected significance threshold of P=${prob_thresh} \[SPM2\]."
    set rs "$rs\[SPM2\] K.J. Friston, A.P. Holmes, K.J. Worsley, J.P. Poline, C.D. Frith and R.S.J. Frackowiak. Statistical Parametric Maps in Functional Imaging: A General Linear Approach. Human Brain Mapping 1995:2(189-210).<br>
"

    set prob_thresh [expr $prob_thresh * 2]
    if {$prob_thresh > 0.99} {
	set prob_thresh 0.99
    }
}

#}}}
	    #{{{ Do the correction

if { $MEDXV == 3.2 } {
    MxClusterDetection $ZS($i) $Mask 1 0.5 "/tmp/${USER}${PROCID}_cluster.txt" I5($i) I6($i) I7($i)
    set X_Resels [exec grep FWHMx /tmp/${USER}${PROCID}_cluster.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    set Y_Resels [exec grep FWHMy /tmp/${USER}${PROCID}_cluster.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    set Z_Resels [exec grep FWHMz /tmp/${USER}${PROCID}_cluster.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    puts "X_Resels=$X_Resels Y_Resels=$Y_Resels Z_Resels=$Z_Resels"
    puts $logout "X_Resels=$X_Resels Y_Resels=$Y_Resels Z_Resels=$Z_Resels"

    set errrr [catch {exec /bin/rm -f /tmp/${USER}${PROCID}_cluster.txt} junk]
    set errrr [catch {exec /bin/rm -f /tmp/${USER}${PROCID}_cluster.txt} junk]
    set errrr [catch {exec /bin/rm -f /tmp/${USER}${PROCID}_cluster.txt} junk]

    MxComputeCorrectedCriticalThr $Mask R $X_Resels $Y_Resels $Z_Resels $prob_thresh outputp zcorrected zuncorrected count
} else {
    MxComputeCorrectedCriticalThr $Mask RS $ZS($i) 2 $prob_thresh outputp zcorrected zuncorrected count
}

puts "outputp=$outputp zcorrected=$zcorrected zuncorrected=$zuncorrected count=$count"
puts $logout "outputp=$outputp zcorrected=$zcorrected zuncorrected=$zuncorrected count=$count"

#}}}
	    #{{{ Use corrected Z threshold to mask Z image

MxMapPixels $ZS($i) -50 $zcorrected 0 0

MxQuietStatistics $ZS($i) { Maximum } false Stats
set MaximumZ($i) [ keylget Stats Maximum ]
puts "MaximumZ =$MaximumZ($i) after thresholding"
puts $logout "MaximumZ =$MaximumZ($i) after thresholding"

#}}}
	} else {
	    # cluster thresholding
	    #{{{ paragraph preparation

if { $i == 1 } {
    set ps "$ps using clusters determined by Z>${z_thresh} and a cluster significance threshold of P=${prob_thresh} \[SPM3\]."
    set rs "$rs\[SPM3\] K.J. Friston, K.J. Worsley, R.S.J. Frackowiak, J.C. Mazziotta and A.C. Evans. Assessing the Significance of Focal Activations Using their Spatial Extent. Human Brain Mapping 1994:1(214-220).<br>
"
}

#}}}
	    if { $MaximumZ($i) > $z_thresh } {
		#{{{ Cluster detection (to get resel sizes)

if { $MEDXV == 3.2 } {
    # for some reason editor popup doesn't work with full path name
    MxClusterDetection $ZS($i) $Mask $z_thresh $prob_thresh /tmp/${USER}${PROCID}_cluster.txt I5($i) I6($i) I7($i)
    set junk [exec /bin/mv /tmp/${USER}${PROCID}_cluster.txt ${CLUSTER}_${stats_name_fixed}.txt]

    set X_Resels [exec grep FWHMx ${CLUSTER}_${stats_name_fixed}.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    set Y_Resels [exec grep FWHMy ${CLUSTER}_${stats_name_fixed}.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    set Z_Resels [exec grep FWHMz ${CLUSTER}_${stats_name_fixed}.txt | grep PHYSICAL | awk {{ print $5 }} - ]
    puts "X_Resels=$X_Resels Y_Resels=$Y_Resels Z_Resels=$Z_Resels"
    puts $logout "X_Resels=$X_Resels Y_Resels=$Y_Resels Z_Resels=$Z_Resels"
} else {
    MxClusterDetection $ZS($i) $Mask S $ZS($i) $z_thresh $prob_thresh /tmp/${USER}${PROCID}_cluster.txt I5($i) I6($i) I7($i) 1
    set junk [exec /bin/mv /tmp/${USER}${PROCID}_cluster.txt ${CLUSTER}_${stats_name_fixed}.txt]
}

#}}}
		#{{{ Use cluster mask to mask Z image

MxDupPage $I5($i) ClusterMask($i)
MxThreshold $ClusterMask($i) 0.5 B1
MxExpr $ZS($i) = " $ZS($i) * $ClusterMask($i) "

#}}}
	    } else {
		MxExpr $ZS($i) = 0
	    }
	    #{{{ report max Z after thresh

MxQuietStatistics $ZS($i) { Maximum } false Stats
set MaximumZ($i) [ keylget Stats Maximum ]
puts "MaximumZ =$MaximumZ($i) after thresholding"
puts $logout "MaximumZ =$MaximumZ($i) after thresholding"

#}}}
	    #{{{ save cluster and probability masks if required

if { $MaximumZ($i) > 0.02 } {
    MxSaveImageAs $I5($i) AVW ${FeatDirName}/cluster_mask_${stats_name_fixed}.hdr true
    MxSaveImageAs $I6($i) AVW ${FeatDirName}/prob_mask_${stats_name_fixed}.hdr true
}

#}}}
	}
    }
}

#}}}
	#{{{ Find group Z min and max

if { $MaximumZ($i) > 0 && $zdisplay == 0 } {
    
    MxQuietStatistics $ZS($i) { Minimum } false 0.002 Stats

    set zminl [ expr [ keylget Stats Minimum ] - 0.002 ]
    if { $zminl < 0.002 } {
	set zminl 0.002
    }

    if { $zmin > $zminl } {
	set zmin $zminl
    }

    if { $zmax < $MaximumZ($i) } {
	set zmax $MaximumZ($i)
    }
}

#}}}
	#{{{ save Z score to file

MxSaveImageAs $ZS($i) AVW ${FeatDirName}/thresh_${stats_name_fixed}.hdr true

#}}}
    }
    
    if { $thresh > 0 } {
	#{{{ Rendering

set RenderList ""
set rendered 0

for { set i 1 } { $i <= $ncon } { incr i 1 } {
    #{{{ Colour rendering

if { $MaximumZ($i) > 0 } {
    incr rendered 1
    set thedate [exec date]
    puts "Starting rendering at $thedate"
    puts $logout "Starting rendering at $thedate"
        
    puts "Rendering using zmin=$zmin zmax=$zmax"
    puts $logout "Rendering using zmin=$zmin zmax=$zmax"
    
    MxQuietStatistics $target_vol { Minimum Maximum } false Stats
    set origmin [ keylget Stats Minimum ]
    set origmax [ keylget Stats Maximum ]
	
    render_proc $F0 $rendertype 0 1 "$target_vol $origmin $origmax $ZS($i) $zmin $zmax"
    
    MxGetCurrentPage R($i)
    lappend RenderList $R($i)

    MxGetImageProperties $R($i) render_props
    set render_name [keylget render_props Name]
    regsub -all " " $render_name "_" render_name_fixed
        
    print_gif [ file rootname [ new_filename ${FeatDirName}/${render_name_fixed}.gif ] ]
    
    #{{{ save to file

    MxSaveImageAs $R($i) VEST ${FeatDirName}/${render_name_fixed}.vest true

#}}}
}

#}}}
}

if { $rendered > 1 } {
    MxSetCurrentPage $FirstPage
    MxGroupVolumes { $RenderList } "Rendered Z statistics" FinalGroup
}

#}}}
    }
}

#}}}
    #{{{ Time series plots and report output for each contrast

if { $paradigm_type == 3 } {
    set fmrifile ${FeatDirName}/filtered_func_data.hdr
    if { ! [ file exists $fmrifile ] } {
	set fmrifile [ string trimright [ file root ${FeatDirName} ] + ].hdr
    }

    set thecommand "exec ${FSLDIR}/bin/tsplot ${FeatDirName} -f $fmrifile -p \"${PLUSES}\""
    puts $thecommand
    set result [ catch { eval $thecommand } ErrMsg ]
}

if { $poststats_yn } {
    puts $report "<hr><H2>Thresholded activation images</H2>"
    for { set i 1 } { $i <= $ncon } { incr i 1 } {
	if { [ file exists ${FeatDirName}/Rendered_thresh_zstat${PLUSES}${i}.gif ] } {
	    puts $report "<p>Contrast $i<br>
	    <a href=\"cluster_zstat${PLUSES}${i}.txt\"><IMG BORDER=0 SRC=\"Rendered_thresh_zstat${PLUSES}${i}.gif\" WIDTH=\"600\"></a>
	    "
	}
    }
}

if { [ file exists ${FeatDirName}/tsplot${PLUSES}1.gif ] } {
    puts $report "<hr><H2>Time series plots</H2><p>"
    for { set i 1 } { $i <= $ncon } { incr i 1 } {
	puts $report "<a href=\"tsplot${PLUSES}${i}.txt\"><IMG BORDER=0 SRC=\"tsplot${PLUSES}${i}.gif\"></a><br>
	"

	if { [ file exists ${FeatDirName}/tsplotc${PLUSES}${i}.gif ] } {
	    puts $report "<a href=\"tsplotc${PLUSES}${i}.txt\"><IMG BORDER=0 SRC=\"tsplotc${PLUSES}${i}.gif\"></a><br>
	    "
	}
    }
}

#}}}
    #{{{ Save folder

set thedate [exec date]
puts "Finished processing at $thedate"
puts $logout "Finished processing at $thedate"

if { $analysis != 4 && !$saveprep_yn } {
    exec /bin/rm -f ${FeatDirName}/filtered_func_data.hdr ${FeatDirName}/filtered_func_data.img
}

if { $savefolder_yn } {
    MxSaveFolder $F0
}

#}}}
    #{{{ Timing and final outputs

MxShowInformationalPopups 1

set thedate [exec date]

puts "Finished FEAT at $thedate"
puts $logout "Finished FEAT at $thedate"

puts $report "$ps

$rs

"
if { $paradigm_type == 3 } {
    puts $report "<p><p><b>Design matrix</a><br><a href=\"design${PLUSES}.mat\"><IMG BORDER=0 SRC=\"design${PLUSES}.gif\"></a>
    "
}

puts $report "<HR><FONT SIZE=1>This page produced automatically by FEAT $VERSION - a part of <A HREF=\"http://www.fmrib.ox.ac.uk/fsl\">FSL - FMRIB's Software Library</A>.</FONT>

</BODY></HTML>
"

close $logout
close $report

puts "
To view the FEAT report point your web browser at
${FeatDirName}/report${PLUSES}.html"

#}}}
}
