#

# FLIRT - FMRIB's Linear Image Registration Tool

# Mark Jenkinson and Stephen Smith, FMRIB Image Analysis Group

# Copyright (C) 1999-2000 University of Oxford

#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA


if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

proc flirt { w } {

    global reg USER FSLDIR

    set reg($w,maxnstats) 20

    MxGetCurrentFolder Folder

    # ---- Set up Frames ----
    toplevel $w
    wm title $w "FLIRT - FMRIB's Linear Image Registration Tool - v1.0"
    wm iconname $w "FLIRT"
    wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm
    tixBalloon    $w.bhelp
    frame $w.f

    tixLabelFrame $w.f.basic
    set lfbasic [ $w.f.basic subwidget frame ]

    tixLabelFrame $w.f.stats
    set lfstats [ $w.f.stats subwidget frame ]
    tixControl $w.f.nstats -label "Number of Secondary Images/Groups to apply transform to " \
	-variable reg($w,nstats) -step 1 -min 0 -max $reg($w,maxnstats) -command "flirt:updatestats $w"
    set reg($w,nstats) 0
    pack $w.f.nstats -in $lfstats -side top -anchor w -pady 3 -padx 5

    frame $w.f.refbox
    label $w.f.reftxt -width 22 -text  "Standard Image"
    entry $w.f.refvar -textvariable reg($w,refimg) -width 45
    pack $w.f.reftxt $w.f.refvar -in $w.f.refbox -side left

    frame $w.f.testbox
    label $w.f.testtxt -width 22 -text "Reslice Image/Group"
    entry $w.f.testvar -textvariable reg($w,testimg) -width 45
    pack $w.f.testtxt $w.f.testvar -in $w.f.testbox -side left

    frame $w.f.testbox2
    label $w.f.testtxt2 -width 22 -text "Low Res Image/Group"
    entry $w.f.testvar2 -textvariable reg($w,testimg2) -width 45
    pack $w.f.testtxt2 $w.f.testvar2 -in $w.f.testbox2 -side left

    # ---- mode ----
    set reg($w,mode) 1
    tixOptionMenu $w.f.mode -label "Mode " \
	    -variable reg($w,mode) \
	    -command "flirt:updatemode $w" \
	    -options {
	label.anchor w
    }
    $w.f.mode add command 1 -label "Reslice Image/Group -> Standard Image"
    $w.f.mode add command 2 -label "Low Res Image/Group -> High Res Image/Group -> Standard Image"
    #$w.bhelp bind $w.f.mode -msg "fsdfds"

    pack $w.f.mode $w.f.refbox $w.f.testbox -in $lfbasic -side top -anchor w -pady 3 -padx 5

    set i 1
    while { $i <= $reg($w,maxnstats) } {
	frame $w.f.secondbox($i)
	label $w.f.secondtxt($i) -width 25 -text "Secondary Image/Group $i"
	entry $w.f.secondvar($i) -textvariable reg($w,secondimg,$i) -width 45
	pack $w.f.secondtxt($i) $w.f.secondvar($i) -in $w.f.secondbox($i) -side left
	incr i 1
    }

    tixLabelFrame $w.f.doff
    set lfdoff [ $w.f.doff subwidget frame ]

    tixOptionMenu $w.f.dof -label "Model/DOF " \
	    -variable reg($w,dof) \
	    -options {
	label.anchor w
	menubutton.width 30
    }
    $w.f.dof add command 3D -label "         3D Models" -state disabled
    $w.f.dof add command 6 -label "Rigid Body (6 parameter model)"
    $w.f.dof add command 7 -label "Global Rescale (7 parameter model)"
    $w.f.dof add command 9 -label "Traditional (9 parameter model)"
    $w.f.dof add command 12 -label "Affine (12 parameter model)"
    set reg($w,dof) 12

    pack $w.f.dof -in $lfdoff -side top -anchor w -padx 5 -pady 1

    pack $w.f.basic $w.f.stats $w.f.doff -in $w.f -side top -anchor w -pady 0 -padx 5
    

    # ---- Optional stuff ----

    collapsible frame $w.f.opts -title "Advanced Options"    

    tixNoteBook $w.nb -ipadx 5 -ipady 5

    $w.nb add search -label "Search"
    $w.nb add anglerep -label "Angle Rep."
    $w.nb add cost -label "Cost Function"
    
    # ---- Search ----
    set lf [$w.nb subwidget search]

    frame $w.search
    frame $w.searchf
    label $w.searchf.angleslabel -text  "Search Angles"

    frame $w.searchf.rx
    tixControl $w.searchf.rxmin -label "X-axis (degrees): min " \
	    -variable reg($w,searchrxmin) -step 1 -min -180 -max 180 \
	    -selectmode immediate
    
    tixControl $w.searchf.rxmax -label "  max" -variable reg($w,searchrxmax) \
	    -step 1 -min -180 -max 180 -selectmode immediate
    pack $w.searchf.rxmin $w.searchf.rxmax -in $w.searchf.rx  -side left

    frame $w.searchf.ry
    tixControl $w.searchf.rymin -label "Y-axis (degrees): min " \
	    -variable reg($w,searchrymin) -step 1 -min -180 -max 180 \
	    -selectmode immediate
    tixControl $w.searchf.rymax -label "  max" -variable reg($w,searchrymax) \
	    -step 1 -min -180 -max 180 -selectmode immediate
    pack $w.searchf.rymin $w.searchf.rymax -in $w.searchf.ry  -side left


    frame $w.searchf.rz
    tixControl $w.searchf.rzmin -label "Z-axis (degrees): min " \
	    -variable reg($w,searchrzmin) -step 1 -min -180 -max 180 \
	    -selectmode immediate
    tixControl $w.searchf.rzmax -label "  max" -variable reg($w,searchrzmax) \
	    -step 1 -min -180 -max 180 -selectmode immediate
    pack $w.searchf.rzmin $w.searchf.rzmax -in $w.searchf.rz  -side left

    global reg($w,search)
    tixOptionMenu $w.search.range -label "Images: " -variable reg($w,search) -command "flirt:updatesearch $w"
    $w.search.range add command 0 -label "Already virtually aligned (no search)"
    $w.search.range add command 1 -label "Not aligned, but same orientation"
    $w.search.range add command 2 -label "Incorrectly oriented"
    set reg($w,search) 1
    set reg($w,disablesearch_yn) 0

    pack $w.searchf.angleslabel $w.searchf.rx $w.searchf.ry $w.searchf.rz -in $w.searchf -side top \
	    -anchor w -padx 3 -pady 3

    pack $w.search.range $w.searchf -in $w.search -side top -anchor w -padx 3 -pady 3

    pack $w.search -in $lf -side top -anchor w

    set reg($w,searchrxmin) -90
    set reg($w,searchrxmax) 90
    set reg($w,searchrymin) -90
    set reg($w,searchrymax) 90
    set reg($w,searchrzmin) -90
    set reg($w,searchrzmax) 90


    # ---- Angle Representation ----
    set lf [$w.nb subwidget anglerep]

    label $w.anglebanner -text "Internal Angle Representation"
    radiobutton $w.euler -text "Euler Angles" \
	    -variable reg($w,anglerep) -value euler

    radiobutton $w.quaternion -text "Quaternions" \
	    -variable reg($w,anglerep) -value quaternion


    # ---- pack ----
    pack $w.anglebanner $w.euler $w.quaternion -in $lf -side top -padx 3 -pady 3
    set reg($w,anglerep) euler



    # ---- Cost Function ----
    set costlf [$w.nb subwidget cost]

    radiobutton $w.corratio -text "Correlation Ratio" \
	    -variable reg($w,cost) -value corratio -anchor w -command "flirt:updatecost $w $costlf"
    radiobutton $w.mutualinfo -text "Mutual Information" \
	    -variable reg($w,cost) -value mutualinfo -anchor w -command "flirt:updatecost $w $costlf"
    radiobutton $w.woods -text "Woods Function" \
	    -variable reg($w,cost) -value woods -anchor w -command "flirt:updatecost $w $costlf"
    radiobutton $w.normcorr -text "Normalised Correlation" \
	    -variable reg($w,cost) -value normcorr -anchor w -command "flirt:updatecost $w $costlf"

    tixControl $w.bins -label "Number of Histogram Bins " \
	    -variable reg($w,bins) -step 1 -min 1 -max 5000 -selectmode immediate
    set reg($w,bins) 256
    
    # ---- pack ----
    pack $w.corratio $w.mutualinfo $w.woods $w.normcorr $w.bins -in $costlf -side top -anchor w -padx 3
    set reg($w,cost) corratio

    frame $w.f.advopts
    pack $w.nb -in $w.f.advopts -side top
    pack $w.f.advopts -in $w.f.opts.b -side left -padx 8 -pady 6 -expand yes -fill both


    pack $w.f.opts -in $w.f -side left



    # ---- Button Frame ----

    frame $w.btns
    frame $w.btns.b -relief raised -borderwidth 1
    
    button $w.ok \
	    -text "OK" -width 5 \
	    -command "flirt:apply $w destroy"
    bind $w.ok <Return> {
        [winfo toplevel %W].ok invoke
    }
    
    button $w.apply     -command "flirt:apply $w keep" \
	    -text "Apply" -width 5
    bind $w.apply <Return> {
	[winfo toplevel %W].apply invoke
    }

    button $w.help -command "FmribWebHelp file: ${FSLDIR}/doc/flirt/index.html" \
	    -text "Help" -width 5
  
    button $w.cancel    -command "flirt:destroy $w" \
	    -text "Cancel" -width 5
    bind $w.cancel <Return> {
	[winfo toplevel %W].cancel invoke
    }
    

    # ---- pack ----
    pack $w.btns.b -side bottom -fill x
    pack $w.ok $w.apply $w.help $w.cancel -in $w.btns.b \
	    -side left -expand yes -padx 3 -pady 10 -fill y
    
    pack $w.f $w.btns -expand yes -fill both
    
}

proc flirt:apply { w dialog } {
    global reg

    set statslist ""
    set i 1
    while { $i <= $reg($w,maxnstats) } {
	lappend statslist $reg($w,secondimg,$i)
	incr i 1
    }

    set status [ flirt_proc $reg($w,mode) $reg($w,refimg) $reg($w,testimg) $reg($w,testimg2) $reg($w,nstats) $statslist $reg($w,dof) $reg($w,bins) $reg($w,searchrxmin) $reg($w,searchrxmax) $reg($w,searchrymin) $reg($w,searchrymax) $reg($w,searchrzmin) $reg($w,searchrzmax) $reg($w,disablesearch_yn) $reg($w,anglerep) $reg($w,cost) 1 ]

    update idletasks
    
    # destroy if the OK button was used AND a normal exit occurred
    if { $dialog == "destroy" && $status == 0 } {
	flirt:destroy $w
    }
}

proc flirt:destroy { w } {
    destroy $w
}

proc flirt:updatemode { w junk } {
    global reg

    if { $reg($w,mode) == 1 } {
	$w.f.testtxt configure -text "Reslice Image/Group"
	$w.f.nstats configure -label "Number of Secondary Images/Groups to apply transform to "
	pack forget $w.f.testbox2
    } else {
	$w.f.testtxt configure -text "High Res Image/Group"
	$w.f.nstats configure -label "Number of Secondary Images/Groups to apply combined transform to "
	pack $w.f.testbox2 -in $w.f -side top -anchor w -pady 3 -padx 5 -after $w.f.testbox
    }
}

proc flirt:updatestats { w junk } {
    global reg

    set i 1
    while { $i <= $reg($w,maxnstats) } {
	pack forget $w.f.secondbox($i)
	incr i 1
    }

    if { $reg($w,nstats) > 0 } {
	pack $w.f.secondbox(1) -in $w.f -side top -anchor w -pady 3 -padx 5 -after $w.f.nstats
	$w.f.secondtxt(1) configure -text "Secondary Image/Group"
    }
    set i 2
    while { $i <= $reg($w,nstats) } {
	$w.f.secondtxt(1) configure -text "Secondary Image/Group 1"
	pack $w.f.secondbox($i) -in $w.f -side top -anchor w -pady 3 -padx 5 -after $w.f.secondbox([ expr $i - 1 ])
	incr i 1
    }
}

proc flirt:updatesearch { w dummy } {
    global reg

    if { $reg($w,search) == 0 } {
	set reg($w,disablesearch_yn) 1
    } else {
	set reg($w,disablesearch_yn) 0

	if { $reg($w,search) == 1 } {
	    set reg($w,searchrxmin) -90
	    set reg($w,searchrxmax) 90
	    set reg($w,searchrymin) -90
	    set reg($w,searchrymax) 90
	    set reg($w,searchrzmin) -90
	    set reg($w,searchrzmax) 90
	} else {
	    set reg($w,searchrxmin) -180
	    set reg($w,searchrxmax) 180
	    set reg($w,searchrymin) -180
	    set reg($w,searchrymax) 180
	    set reg($w,searchrzmin) -180
	    set reg($w,searchrzmax) 180
	}
    }




    if { $reg($w,disablesearch_yn) } {
	pack forget $w.searchf
    } else {
	pack $w.searchf -in $w.search -side top -anchor w -padx 3 -pady 3
    }
}

proc flirt:updatecost { w costlf } {
    global reg

    if { [ string match $reg($w,cost) "normcorr" ] == 1 } {
	pack forget $w.bins
    } else {
	pack $w.bins -in $costlf -side top -anchor w -padx 3
    }
}

flirt .rename
tkwait window .rename
