#!/usr/freeware/medx/bin/tixwish

# FLIRT - FMRIB's Linear Image Registration Tool

# Mark Jenkinson and Stephen Smith, FMRIB Image Analysis Group

# Copyright (C) 1999-2000 University of Oxford

#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

proc FixMedxTransform { xfmfname OriginX OriginY OriginZ XPixelSeparation YPixelSeparation ZPixelSeparation } {

    set notfound 0
    set optr [ open ${xfmfname}TMP w ]
    set fptr [ open $xfmfname r ]

    while { ( $notfound == 0 ) && ( [ gets $fptr line ] >= 0 ) } {
	if { [ regexp outputusermatrix $line ] } {
	    set notfound 1
	}
	puts $optr $line
    }

    puts $optr "            $XPixelSeparation"
    puts $optr "            0"
    puts $optr "            0"
    puts $optr "            $OriginX"
    puts $optr "            0"
    puts $optr "            $YPixelSeparation"
    puts $optr "            0"
    puts $optr "            $OriginY"
    puts $optr "            0"
    puts $optr "            0"
    puts $optr "            $ZPixelSeparation"
    puts $optr "            $OriginZ"
    puts $optr "            0"
    puts $optr "            0"
    puts $optr "            0"
    puts $optr "            1"

    for { set i 0 } { $i < 16 } { incr i 1 } {
	gets $fptr line
    }

    while { [ gets $fptr line ] >= 0 } {
	puts $optr $line
    }
    
    close $fptr
    close $optr

    set errrr [ catch { exec /bin/mv -f ${xfmfname}TMP ${xfmfname} } junk ]    
}

proc TalairachMedxTransform { xfmfname } {

    set optr [ open ${xfmfname}TMP w ]
    set fptr [ open $xfmfname r ]

    while { [ gets $fptr line ] >= 0 } {
	if { [ regexp talairachcalibrated $line ] } {
	    puts $optr "        /talairachcalibrated?	true"
	} else {
	    puts $optr $line
	}
    }
    
    close $fptr
    close $optr

    set errrr [ catch { exec /bin/mv -f ${xfmfname}TMP ${xfmfname} } junk ]    
}

proc ConcatedTalairachMedxTransform { xfmfname } {

    set optr [ open ${xfmfname}TMP w ]
    set fptr [ open $xfmfname r ]
    set notfixed 1

    while { [ gets $fptr line ] >= 0 } {
	if { $notfixed && [ regexp ">>" $line ] } {
	    puts $optr "        /talairachcalibrated?	true"
	    set notfixed 0
	}
	puts $optr $line
    }
    
    close $fptr
    close $optr

    set errrr [ catch { exec /bin/mv -f ${xfmfname}TMP ${xfmfname} } junk ]    
}

proc flirt_proc { regmode refname testname testname2 nstats statslist dof bins searchrxmin searchrxmax searchrymin searchrymax searchrzmin searchrzmax disablesearch_yn anglerep cost popups } {

    # setup vars
    global PXHOME USER PROCID MEDXV HOME

    set TempFileBase "${HOME}/.flirt_${PROCID}_reg"

    MxGetCurrentFolder Folder
    if { ! [ info exists Folder ] } {
	if { $popups } {
	    LocalErrorBox "A folder must be opened!"
	}
	puts "A folder must be opened!"
	return 1
    }

    set problem [ MxGetPageByName $Folder $refname refptr ]
    if { $problem == 1 } {
	if { $popups } {
	    LocalErrorBox "Standard Image is not a valid page!"
	}
	puts "Standard Image is not a valid page!"
	return 1
    }
    MxGetPageProperties $refptr Props
    if { [ string match [ keylget Props Type ] Volume ] != 1 } {
	if { $popups } {
	    LocalErrorBox "Standard Image isn't a volume!"
	}
	puts "Standard Image isn't a volume!"
	return 1
    }

    set problem [ MxGetPageByName $Folder $testname testptr ]
    if { $problem == 1 } {
	if { $popups } {
	    LocalErrorBox "Reslice Image/Group is not a valid page!"
	}
	puts "Reslice Image/Group is not a valid page!"
	return 1
    }
    MxSetCurrentPage $testptr
    set testtype [ what_type 0 ]
    if { $testtype != 3 && $testtype != 4 } {
	if { $popups } {
	    LocalErrorBox "Reslice Image/Group isn't a volume or a group!"
	}
	puts "Reslice Image/Group isn't a volume or a group!"
	return 1
    }
    if { $testtype == 3 } {
	set testpages $testptr
	set testvolumes 1
    } else {
	MxGetPagesFromGroup $testptr testpages
	set testvolumes [ llength $testpages ]
    }

    # check secondary (stats) group
    if { $nstats > 0 } {
	set i 1
	while { $i <= $nstats } {
	    set problem [ MxGetPageByName $Folder [ lindex $statslist [ expr $i - 1 ] ] secondptr($i) ]
	    if { $problem == 1 } {
		if { $popups } {
		    LocalErrorBox "Secondary Image/Group(s) not valid page(s)!"
		}
		puts "Secondary Image/Group(s) not valid page(s)!"
		return 1
	    } else {
		MxSetCurrentPage $secondptr($i)
		set secondtype [ what_type 0 ]
		if { $testtype != $secondtype } {
		    if { $popups } {
			LocalErrorBox "All Secondary and High Res entries must either be single volumes or be groups!"
		    }
		    puts "All Secondary and High Res entries must either be single volumes or be groups!"
		    return 1
		}

		if { $testtype == 3 } {
		    set secondpages($i) $secondptr($i)
		    set secondvolumes 1
		} else {
		    MxGetPagesFromGroup $secondptr($i) secondpages($i)
		    set secondvolumes [ llength $secondpages($i) ]
		}
		
		if { $testvolumes != $secondvolumes } {
		    if { $popups } {
			LocalErrorBox "Secondary Image/Group doesn't contain the right number of images!"
		    }
		    puts "Secondary Image/Group doesn't contain the right number of images!"
		    return 1
		}
	    }
	    incr i 1
	}
    }

    # check test2 (Low Res) group
    if { $regmode == 2 } {
	set problem [ MxGetPageByName $Folder $testname2 testptr2 ]
	if { $problem == 1 } {
	    if { $popups } {
		LocalErrorBox "Low Res Image/Group is not a valid page!"
	    }
	    puts "Low Res Image/Group is not a valid page!"
	    return 1
	} else {
	    MxSetCurrentPage $testptr2
	    set testtype2 [ what_type 0 ]
	    if { $testtype != $testtype2 } {
		if { $popups } {
		    LocalErrorBox "Low Res and High Res entries must either both be single volumes or both be groups!"
		}
		puts "Low Res and High Res entries must either both be single volumes or both be groups!"
		return 1
	    }

	    if { $testtype == 3 } {
		set testpages2 $testptr2
		set testvolumes2 1
	    } else {
		MxGetPagesFromGroup $testptr2 testpages2
		set testvolumes2 [ llength $testpages2 ]
	    }

	    if { $testvolumes != $testvolumes2 } {
		if { $popups } {
		    LocalErrorBox "Low Res Image/Group doesn't contain the right number of images!"
		}
		puts "Low Res Image/Group doesn't contain the right number of images!"
		return 1
	    }
	}
    }

    # Image/Group 1 to Standard
    set i 0
    set NewList ""
    foreach testpage $testpages {
	set returnval [ flirt_run $Folder $refptr $testpage $dof $bins $searchrxmin $searchrxmax $searchrymin $searchrymax $searchrzmin $searchrzmax $disablesearch_yn $anglerep $cost $popups ${TempFileBase}_${i} ]
	MxGetCurrentPage output
	lappend NewList $output
	incr i 1
    }
    if { $testtype == 4 } {
	MxGroupVolumes $NewList "Registered Group" OutputGroup
    }

    # Image/Group 2 to Image/Group 1 and the apply concated transforms to group 2
    if { $regmode == 2 } {

	set i 0
	set NewList ""
	set NewList2 ""
	foreach testpage2 $testpages2 {

	    set refptr2 [ lindex $testpages $i ]
	    set returnval [ flirt_run $Folder $refptr2 $testpage2 $dof $bins $searchrxmin $searchrxmax $searchrymin $searchrymax $searchrzmin $searchrzmax $disablesearch_yn $anglerep $cost $popups ${TempFileBase}_level2_${i} ]
	    MxGetCurrentPage output
	    lappend NewList $output
	    MxConcatTransforms ${TempFileBase}_level2_${i}_result.xfm AIR ${TempFileBase}_${i}_result.xfm AIR ${TempFileBase}_combined_${i}_result.xfm
	    MxGetImageProperties $refptr ImageProps1
	    if { [ regexp true [ keylget ImageProps1 TalairachCalibrated ] ] } {
		ConcatedTalairachMedxTransform ${TempFileBase}_combined_${i}_result.xfm
	    }

	    MxSetCurrentPage $testpage2
	    if { $MEDXV == 3.2 } {
		MxApplySavedTransform $testpage2 ${TempFileBase}_combined_${i}_result.xfm AIR resultptr   
	    } else {
		MxApplySavedTransform $testpage2 ${TempFileBase}_combined_${i}_result.xfm AIR F false resultptr   
	    }
	    MxGetImageProperties $testpage2 ImageProps2
	    name_page $resultptr "Shadow Registration of [keylget ImageProps2 Name] to [keylget ImageProps1 Name]"
	    MxSetCurrentPage $resultptr
	    lappend NewList2 $resultptr

	    incr i 1
	}
	if { $testtype == 4 } {
	    MxGroupVolumes $NewList "Registered Low Res to High Res Group" OutputGroup2
	    MxGroupVolumes $NewList2 "Shadow Registered Low Res Group" ShadowOutputGroup2
	}
    }


    # apply to stats images
    if { $nstats > 0 } {
	set ii 1
	while { $ii <= $nstats } {
	    set i 0
	    set NewList ""
	    foreach secondpage $secondpages($ii) {
		MxSetCurrentPage $secondpage
		if { $regmode == 1 } {
		    set xfmname ${TempFileBase}_${i}_result.xfm
		} else {
		    set xfmname ${TempFileBase}_combined_${i}_result.xfm
		}
		if { $MEDXV == 3.2 } {
		    MxApplySavedTransform $secondpage $xfmname AIR resultptr   
		} else {
		    MxApplySavedTransform $secondpage $xfmname AIR F false resultptr   
		}
		MxGetImageProperties $refptr ImageProps1
		MxGetImageProperties $secondpage ImageProps2
		name_page $resultptr "Shadow Registration of [keylget ImageProps2 Name] to [keylget ImageProps1 Name]"
		MxSetCurrentPage $resultptr
		lappend NewList $resultptr
		incr i 1
	    }
	    if { $testtype == 4 } {
		MxGroupVolumes $NewList "Shadow Registered Group $ii" ShadowOutputGroup($ii)
	    }
	    incr ii 1
	}
    }

    return $returnval
}

proc flirt_run { Folder refptr testptr dof bins searchrxmin searchrxmax searchrymin searchrymax searchrzmin searchrzmax disablesearch_yn anglerep cost popups TempFileBase } {

    # setup vars
    global FSLDIR MEDXV CLUSTERRSH
    
    # set up file names for stored images and results
    set reffile      ${TempFileBase}_ref.hdr
    set testfile     ${TempFileBase}_test.hdr
    set xfmfilename  ${TempFileBase}_result.xfm
    set txtfile      ${TempFileBase}.txt

    # save images and fix permissions
    MxSaveImageAs $refptr AVW ${reffile} true
    MxSaveImageAs $testptr AVW ${testfile} true
    set errrr [ catch { exec /bin/chmod 755 [ file rootname $reffile ].hdr } junk ]
    set errrr [ catch { exec /bin/chmod 755 [ file rootname $reffile ].img } junk ]
    set errrr [ catch { exec /bin/chmod 755 [ file rootname $testfile ].hdr } junk ]
    set errrr [ catch { exec /bin/chmod 755 [ file rootname $testfile ].img } junk ]

    # set up the command to be executed
    set basecommand "$CLUSTERRSH ${FSLDIR}/bin/flirt"
    set fullcommand "exec $basecommand"
    set fullcommand "$fullcommand -ref $reffile -in $testfile"
    set fullcommand "$fullcommand -omedx $xfmfilename"
    set fullcommand "$fullcommand -dof $dof"
    set fullcommand "$fullcommand -bins $bins"
    set fullcommand "$fullcommand -cost $cost"
    set fullcommand "$fullcommand -anglerep $anglerep"
    set fullcommand "$fullcommand -v -nosave"
    if { $disablesearch_yn } {
	set fullcommand "$fullcommand -searchrx 0 0 -searchry 0 0 -searchrz 0 0"
    } else {
	set fullcommand "$fullcommand -searchrx $searchrxmin $searchrxmax"
	set fullcommand "$fullcommand -searchry $searchrymin $searchrymax"
	set fullcommand "$fullcommand -searchrz $searchrzmin $searchrzmax"
    }

    set fullcommand "$fullcommand >& $txtfile"

    set thedate [exec date]
    puts "Starting FLIRT at $thedate"

    # run command
    ScriptUpdate "Calling $basecommand: As this process is not fully integrated into the heart of MEDx, the traffic lights will not show up red during processing. Wait for this popup to disappear before continuing."
    puts $fullcommand
    set result [ catch { eval $fullcommand } ErrMsg ]
    CancelScriptUpdate
    if {$result != 0} {
	puts "$ErrMsg"
    }

    set success [ file readable "$xfmfilename" ]
    if { $success == 0 } {
	if { $popups } {
	    LocalErrorBox "No transformation saved!"
	}
	puts "No transformation saved!"
	return 4
    }

    MxGetCoordinateSystem $refptr OriginX OriginY OriginZ XPixelSeparation YPixelSeparation ZPixelSeparation
    FixMedxTransform $xfmfilename $OriginX $OriginY $OriginZ $XPixelSeparation $YPixelSeparation $ZPixelSeparation

    MxGetImageProperties $refptr ImageProps1
    if { [ regexp true [ keylget ImageProps1 TalairachCalibrated ] ] } {
	TalairachMedxTransform $xfmfilename
    }
	
    MxSetCurrentPage $testptr
    if { $MEDXV == 3.2 } {
	MxApplySavedTransform $testptr "$xfmfilename" AIR resultptr   
    } else {
	MxApplySavedTransform $testptr "$xfmfilename" AIR F false resultptr   
    }

    MxGetImageProperties $testptr ImageProps2
    name_page $resultptr "Registration of [keylget ImageProps2 Name] to [keylget ImageProps1 Name]"
    MxSetCurrentPage $resultptr

    # Tidy up temporary files
    set errrr [ catch { exec /bin/rm -f $xfmfilename } junk ]    
    set errrr [ catch { exec /bin/rm -f $txtfile } junk ]    
    set errrr [ catch { exec /bin/rm -f $reffile } junk ]    
    set errrr [ catch { exec /bin/rm -f $testfile } junk ]
    set errrr [ catch { exec /bin/rm -f [ file rootname $reffile ].img } junk ]    
    set errrr [ catch { exec /bin/rm -f [ file rootname $testfile ].img } junk ]    

    # Timing
    set thedate [exec date]
    puts "Finished FLIRT at $thedate"
    return 0
}
