#

#   scatter - creat scatter plot from two images
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA


if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

proc show_graph { Filename } {

    set count 0
    set w ".dialog[incr count]"
    while { [ winfo exists $w ] } {
        set w ".dialog[incr count]"
    }

    toplevel $w
    wm title $w "Scatter Plot"
    wm iconname $w "Plot"

    frame $w.f

    label $w.f.label
    pack $w.f.label -in $w.f
    pack $w.f

    set graphpic [image create photo -file $Filename ]
    $w.f.label configure -image $graphpic
}

proc scatter {Folder Group dummy} {

    global FSLDIR USER PROCID

MxGetPagesFromGroup $Group Pages
puts $Pages

MxGetImageProperties [ lindex $Pages 0 ] props

if { [ keylget props Slices ] == 0 } {
    puts "Will have to force these to be volumes..."
    set thelist ""
    foreach Page $Pages {
	MxComposeVolume { $Page } V
	lappend thelist $V
    }
    MxGroupVolumes { $thelist } "Group of Vols" Group
    MxGetPagesFromGroup $Group Pages
    puts $Pages
}

MxSave3DGroupAs $Group AVW /tmp/${USER}_scatter.hdr true 

set thecommand "exec ${FSLDIR}/bin/scatter /tmp/${USER}_scatter /tmp/${USER}_scatter.txt"
puts $thecommand
ScriptUpdate "Calling $thecommand: As this process is not fully integrated into the heart of MEDx, the traffic lights will not show up red during processing. Wait for this popup to disappear before continuing."
set result [catch {eval $thecommand} ErrMsg]

if {$result != 0} {
  puts "$ErrMsg"
}

MxSetCurrentPage $Group
CancelScriptUpdate
ScriptUpdate "Running gnuplot"

set gpid [ open "| ${FSLDIR}/bin/gnuplot > /tmp/${USER}_gnuplot" "w" ]
            puts $gpid "set term pbm small color"
            puts $gpid "plot \'/tmp/${USER}_scatter.txt\' with dots"
            close $gpid
show_graph /tmp/${USER}_gnuplot

CancelScriptUpdate
MxPause ""
}
