#

#   renderstats_proc.tcl - TCL proc for colour overlay rendering
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

#{{{ example calc calls

# setup mask using min and max
#MxExpr $Page = " ( 1 - $corr * Sqrt ( $adjust(0) - Sqrt( $Page + $adjust(0) - $smin(0) ) ) ) * \
#	( 1 - $corr * Sqrt ( $adjust(0) - Sqrt( $smax(0) - $Page + $adjust(0) ) ) ) "

# crop to min and max
#MxExpr $Page = " Min( Max( $Page , $bmin ) , $bmax ) "

# make integer
# ( ( val << 1 ) >> 1 )

#}}}

proc render_proc { Folder colour_type output_type images imagelist } {
    global FSLDIR

    if { $imagelist == "" } {
	MxGetCurrentImage OUT
    } else {
	#{{{ background setup

set bg [ lindex $imagelist 0 ]
set bmin [ lindex $imagelist 1 ]
set bmax [ lindex $imagelist 2 ]

set IsStats 0

#}}}
	#{{{ stats setup

set finalname "Rendered "

    set i 1
    while { $i <= $images } {
	set si($i) [ lindex $imagelist [ expr $i * 3 ] ]
	set smin($i) [ lindex $imagelist [ expr $i * 3 + 1 ] ]
	set smax($i) [ lindex $imagelist [ expr $i * 3 + 2 ] ]
	set srange($i) [ expr $smax($i) - $smin($i) ]

	MxGetImageProperties $si($i) pl
	set enc($i) [ keylget pl Encoding ]
	set bpp($i) [ keylget pl Bits ]
	MxGetDimensions $si($i) width height depth
	if { $enc($i) == "R" } {
	    set adjust($i) 0.0001
	    if { $bpp($i) == 32 && $width < 150 && $output_type == 0 } {
		set IsStats 1
	    }
	} else {
	    set adjust($i) 1
	}

	if { $i > 1 } {
	    append finalname " and "
	}
	append finalname "[ keylget pl Name ]"
	
	set corr($i) [ expr 1 / sqrt( $adjust($i) ) ]
	incr i
    }

#}}}
	#{{{ Prepare output image

# MxDupPage is used so that the stats information associated with
# $si(1) is carried to OUT, e.g. the time series stuff. In here is a
# cunning bit to save the additional cost of running MxDupPage if we are
# not working with a stats image in the first place.

if { $IsStats } {
    MxDupPage $si(1) OUT
    set calcstring " "
} else {
    set OUT OUT
    if { $output_type == 0 } {
	set calcstring {{Encoding R} {Bits 32} {NewName blah}}
    } else {
	set calcstring {{Encoding SI} {Bits 16} {NewName blah}}
    }
}

#}}}
	#{{{ setup ranges

set A [ expr ( 39.0 * $smin(1) - 21.0 * $smax(1) ) / 18.0 ]
set B [ expr ( $smin(1) - $A ) / 1.05 + $A ]

set hrange [ expr $B - $A ]

if { $output_type == 0 } {
    set multiplier 1.0
    set offset 0.0
} else {
    set multiplier [ expr 32000.0 / ( $hrange * ( 1 + ( $images * $images ) ) ) ]
    set offset [ expr ( 0 - $A ) * $multiplier ]
}

puts " A=$A B=$B hrange=$hrange smin(1)=$smin(1) smax(1)=$smax(1) "

#}}}
	#{{{ 1 stats images

# solid
# OUT = ( rescaled_background * ( 1 - stats_min_mask * stats_max_mask ) ) +
#       ( rescaled_stats * stats_min_mask * stats_max_mask )

    if { $images == 1 } {

	if { $colour_type == 0 } {

	    set calc2string " $offset + $multiplier * (
	        ( ( Min( Max( $bg , $bmin ) , $bmax ) - $bmin ) * ( $hrange * 0.95 / ( $bmax - $bmin ) ) + $A ) * ( 1 - \
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $si(1) + $adjust(1) - $smin(1) ) ) ) * \
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $smax(1) - $si(1) + $adjust(1) ) ) ) ) + \
		$si(1) * ( \
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $si(1) + $adjust(1) - $smin(1) ) ) ) * \
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $smax(1) - $si(1) + $adjust(1) ) ) ) ) \
              )	"

	} else {

	    set calc2string " $offset + $multiplier * (
	        ( ( Min( Max( $bg , $bmin ) , $bmax ) - $bmin ) * ( $hrange * 0.95 / ( $bmax - $bmin ) ) + $A ) * ( 1 -
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $si(1) + $adjust(1) - $smin(1) ) ) ) *
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $smax(1) - $si(1) + $adjust(1) ) ) ) ) +
		( ( ( ( ( ( $si(1) - $smin(1) ) * 5 * 0.999 / $srange(1) ) << 1 ) >> 1 ) * $hrange * 0.2 ) +
                  ( ( Min( Max( $bg , $bmin ) , $bmax ) - $bmin ) * ( $hrange * 0.5 * 0.2 / ( $bmax - $bmin ) ) ) +
                  ( $B + $hrange * 0.25 * 0.2 ) ) * (
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $si(1) + $adjust(1) - $smin(1) ) ) ) *
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $smax(1) - $si(1) + $adjust(1) ) ) ) )
              )	"

	}
    }

#}}}
	#{{{ 2 stats images

# OUT =
# ( (rescaled background) * ( 1 -
#   stats1 min mask *
#   stats1 max mask ) ) * ( 1 -
#   stats2 min mask *
#   stats2 max mask ) ) +
# ( (rescaled stats1) *
#   stats1 min mask *
#   stats1 max mask ) +
# ( (rescaled stats2) *
#   stats2 min mask *
#   stats max mask )

    if { $images == 2 } {

	if { $colour_type == 0 } {

	    set calc2string " $offset + $multiplier * (
                $A + ( ( Min( Max( $bg , $bmin ) , $bmax ) - $bmin ) * ( $hrange * 0.95 / ( $bmax - $bmin ) )  ) * ( 1 -
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $si(1) + $adjust(1) - $smin(1) ) ) ) *
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $smax(1) - $si(1) + $adjust(1) ) ) ) ) * ( 1 -
		( 1 - $corr(2) * Sqrt ( $adjust(2) - Sqrt( $si(2) + $adjust(2) - $smin(2) ) ) ) *
		( 1 - $corr(2) * Sqrt ( $adjust(2) - Sqrt( $smax(2) - $si(2) + $adjust(2) ) ) ) ) +
		( ( ( $si(1) - $smin(1) ) * $hrange * 0.9 / $srange(1) ) + ( $hrange * 1.05 ) ) * (
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $si(1) + $adjust(1) - $smin(1) ) ) ) *
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $smax(1) - $si(1) + $adjust(1) ) ) ) ) +
		( ( ( $si(2) - $smin(2) ) * $hrange * 0.9 / $srange(2) ) + ( $hrange * 2.05 ) ) * (
		( 1 - $corr(2) * Sqrt ( $adjust(2) - Sqrt( $si(2) + $adjust(2) - $smin(2) ) ) ) *
		( 1 - $corr(2) * Sqrt ( $adjust(2) - Sqrt( $smax(2) - $si(2) + $adjust(2) ) ) ) )
                ) "

	} else {

	    set calc2string " $offset + $multiplier * (
                $A + ( ( Min( Max( $bg , $bmin ) , $bmax ) - $bmin ) * ( $hrange * 0.95 / ( $bmax - $bmin ) )  ) * ( 1 -
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $si(1) + $adjust(1) - $smin(1) ) ) ) *
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $smax(1) - $si(1) + $adjust(1) ) ) ) ) * ( 1 -
		( 1 - $corr(2) * Sqrt ( $adjust(2) - Sqrt( $si(2) + $adjust(2) - $smin(2) ) ) ) *
		( 1 - $corr(2) * Sqrt ( $adjust(2) - Sqrt( $smax(2) - $si(2) + $adjust(2) ) ) ) ) +
		( ( ( ( ( ( $si(1) - $smin(1) ) * 5 * 0.999 / $srange(1) ) << 1 ) >> 1 ) * $hrange * 0.2 ) +
                  ( ( Min( Max( $bg , $bmin ) , $bmax ) - $bmin ) * ( $hrange * 0.5 * 0.2 / ( $bmax - $bmin ) ) ) +
                  ( $hrange + $hrange * 0.25 * 0.2 ) ) * (
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $si(1) + $adjust(1) - $smin(1) ) ) ) *
		( 1 - $corr(1) * Sqrt ( $adjust(1) - Sqrt( $smax(1) - $si(1) + $adjust(1) ) ) ) ) +
		( ( ( ( ( ( $si(2) - $smin(2) ) * 5 * 0.999 / $srange(2) ) << 1 ) >> 1 ) * $hrange * 0.2 ) +
                  ( ( Min( Max( $bg , $bmin ) , $bmax ) - $bmin ) * ( $hrange * 0.5 * 0.2 / ( $bmax - $bmin ) ) ) +
                  ( $hrange * 2 + $hrange * 0.25 * 0.2 ) ) * (
		( 1 - $corr(2) * Sqrt ( $adjust(2) - Sqrt( $si(2) + $adjust(2) - $smin(2) ) ) ) *
		( 1 - $corr(2) * Sqrt ( $adjust(2) - Sqrt( $smax(2) - $si(2) + $adjust(2) ) ) ) )
                ) "

	}
    }

#}}}
	#{{{ MxExpr and rename

MxExpr $calcstring $OUT = $calc2string

keylset OProps Name "$finalname"
MxSetImageProperties $OUT $OProps

if { $output_type == 0 } {
    MxSetDisplayRange $OUT V $A [ expr $A + ( $hrange * ( 1 + ( $images * $images ) ) ) ]
} else {
    MxSetDisplayRange $OUT V 0 32000
}

#}}}
    }
    #{{{ LUTS

set typeflag ""
if { $colour_type } {
    set typeflag "t"
}

MxLoadMaterialMap $OUT ${FSLDIR}/tcl/render${images}${typeflag}.map
MxLoadLUT $OUT ${FSLDIR}/tcl/render${images}${typeflag}.lut

#}}}
}
